@@ -77,6 +77,9 @@
  * distinguish them.
  */
 
+int (*igt_ioctl)(int fd, unsigned long request, void *arg) = drmIoctl;
+
+
 /**
  * gem_handle_to_libdrm_bo:
  * @bufmgr: libdrm buffer manager instance
@@ -108,6 +111,19 @@ gem_handle_to_libdrm_bo(drm_intel_bufmgr *bufmgr, int fd, const char *name, uint
 	return bo;
 }
 
+static int
+__gem_get_tiling(int fd, struct drm_i915_gem_get_tiling *arg)
+{
+	int err;
+
+	err = 0;
+	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_GET_TILING, arg))
+		err = -errno;
+	errno = 0;
+
+	return err;
+}
+
 /**
  * gem_get_tiling:
  * @fd: open i915 drm file descriptor
@@ -116,21 +132,23 @@ gem_handle_to_libdrm_bo(drm_intel_bufmgr *bufmgr, int fd, const char *name, uint
  * @swizzle: (out) bit 6 swizzle mode
  *
  * This wraps the GET_TILING ioctl.
+ *
+ * Returns whether the actual physical tiling matches the reported tiling.
  */
-void
+bool
 gem_get_tiling(int fd, uint32_t handle, uint32_t *tiling, uint32_t *swizzle)
 {
 	struct drm_i915_gem_get_tiling get_tiling;
-	int ret;
 
 	memset(&get_tiling, 0, sizeof(get_tiling));
 	get_tiling.handle = handle;
 
-	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_GET_TILING, &get_tiling);
-	igt_assert(ret == 0);
+	igt_assert_eq(__gem_get_tiling(fd, &get_tiling), 0);
 
 	*tiling = get_tiling.tiling_mode;
 	*swizzle = get_tiling.swizzle_mode;
+
+	return get_tiling.phys_swizzle_mode == get_tiling.swizzle_mode;
 }
 
 int __gem_set_tiling(int fd, uint32_t handle, uint32_t tiling, uint32_t stride)
@@ -138,6 +156,12 @@ int __gem_set_tiling(int fd, uint32_t handle, uint32_t tiling, uint32_t stride)
 	struct drm_i915_gem_set_tiling st;
 	int ret;
 
+	igt_require_intel(fd);
+
+	/* The kernel doesn't know about these tiling modes, expects NONE */
+	if (tiling == I915_TILING_Yf || tiling == I915_TILING_Ys)
+		tiling = I915_TILING_NONE;
+
 	memset(&st, 0, sizeof(st));
 	do {
 		st.handle = handle;
@@ -180,6 +204,23 @@ struct local_drm_i915_gem_caching {
 #define LOCAL_DRM_IOCTL_I915_GEM_GET_CACHEING \
 	DRM_IOWR(DRM_COMMAND_BASE + LOCAL_DRM_I915_GEM_GET_CACHEING, struct local_drm_i915_gem_caching)
 
+static int __gem_set_caching(int fd, uint32_t handle, uint32_t caching)
+{
+	struct local_drm_i915_gem_caching arg;
+	int err;
+
+	memset(&arg, 0, sizeof(arg));
+	arg.handle = handle;
+	arg.caching = caching;
+
+	err = 0;
+	if (igt_ioctl(fd, LOCAL_DRM_IOCTL_I915_GEM_SET_CACHEING, &arg)) {
+		err = -errno;
+		igt_assert(errno == ENOTTY || errno == EINVAL);
+	}
+	return err;
+}
+
 /**
  * gem_set_caching:
  * @fd: open i915 drm file descriptor
@@ -193,17 +234,7 @@ struct local_drm_i915_gem_caching {
  */
 void gem_set_caching(int fd, uint32_t handle, uint32_t caching)
 {
-	struct local_drm_i915_gem_caching arg;
-	int ret;
-
-	memset(&arg, 0, sizeof(arg));
-	arg.handle = handle;
-	arg.caching = caching;
-
-	ret = drmIoctl(fd, LOCAL_DRM_IOCTL_I915_GEM_SET_CACHEING, &arg);
-	igt_assert(ret == 0 || (errno == ENOTTY || errno == EINVAL));
-
-	igt_require(ret == 0);
+	igt_require(__gem_set_caching(fd, handle, caching) == 0);
 	errno = 0;
 }
 
@@ -221,8 +252,8 @@ uint32_t gem_get_caching(int fd, uint32_t handle)
 	struct local_drm_i915_gem_caching arg;
 	int ret;
 
+	memset(&arg, 0, sizeof(arg));
 	arg.handle = handle;
-	arg.caching = 0;
 	ret = ioctl(fd, LOCAL_DRM_IOCTL_I915_GEM_GET_CACHEING, &arg);
 	igt_assert(ret == 0);
 	errno = 0;
@@ -298,6 +329,23 @@ void gem_close(int fd, uint32_t handle)
 	do_ioctl(fd, DRM_IOCTL_GEM_CLOSE, &close_bo);
 }
 
+static int __gem_write(int fd, uint32_t handle, uint64_t offset, const void *buf, uint64_t length)
+{
+	struct drm_i915_gem_pwrite gem_pwrite;
+	int err;
+
+	memset(&gem_pwrite, 0, sizeof(gem_pwrite));
+	gem_pwrite.handle = handle;
+	gem_pwrite.offset = offset;
+	gem_pwrite.size = length;
+	gem_pwrite.data_ptr = (uintptr_t)buf;
+
+	err = 0;
+	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_PWRITE, &gem_pwrite))
+		err = -errno;
+	return err;
+}
+
 /**
  * gem_write:
  * @fd: open i915 drm file descriptor
@@ -311,16 +359,25 @@ void gem_close(int fd, uint32_t handle)
  */
 void gem_write(int fd, uint32_t handle, uint64_t offset, const void *buf, uint64_t length)
 {
-	struct drm_i915_gem_pwrite gem_pwrite;
-
-	memset(&gem_pwrite, 0, sizeof(gem_pwrite));
-	gem_pwrite.handle = handle;
-	gem_pwrite.offset = offset;
-	gem_pwrite.size = length;
-	gem_pwrite.data_ptr = (uintptr_t)buf;
-	do_ioctl(fd, DRM_IOCTL_I915_GEM_PWRITE, &gem_pwrite);
+	igt_assert_eq(__gem_write(fd, handle, offset, buf, length), 0);
 }
 
+static int __gem_read(int fd, uint32_t handle, uint64_t offset, void *buf, uint64_t length)
+{
+	struct drm_i915_gem_pread gem_pread;
+	int err;
+
+	memset(&gem_pread, 0, sizeof(gem_pread));
+	gem_pread.handle = handle;
+	gem_pread.offset = offset;
+	gem_pread.size = length;
+	gem_pread.data_ptr = (uintptr_t)buf;
+
+	err = 0;
+	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_PREAD, &gem_pread))
+		err = -errno;
+	return err;
+}
 /**
  * gem_read:
  * @fd: open i915 drm file descriptor
@@ -334,14 +391,7 @@ void gem_write(int fd, uint32_t handle, uint64_t offset, const void *buf, uint64
  */
 void gem_read(int fd, uint32_t handle, uint64_t offset, void *buf, uint64_t length)
 {
-	struct drm_i915_gem_pread gem_pread;
-
-	memset(&gem_pread, 0, sizeof(gem_pread));
-	gem_pread.handle = handle;
-	gem_pread.offset = offset;
-	gem_pread.size = length;
-	gem_pread.data_ptr = (uintptr_t)buf;
-	do_ioctl(fd, DRM_IOCTL_I915_GEM_PREAD, &gem_pread);
+	igt_assert_eq(__gem_read(fd, handle, offset, buf, length), 0);
 }
 
 /**
@@ -390,7 +440,7 @@ int gem_wait(int fd, uint32_t handle, int64_t *timeout_ns)
 	wait.flags = 0;
 
 	ret = 0;
-	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_WAIT, &wait))
+	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_WAIT, &wait))
 		ret = -errno;
 
 	if (timeout_ns)
@@ -424,7 +474,7 @@ bool gem_create__has_stolen_support(int fd)
 
 	if (has_stolen_support < 0) {
 		memset(&gp, 0, sizeof(gp));
-		gp.param = 36; /* CREATE_VERSION */
+		gp.param = 38; /* CREATE_VERSION */
 		gp.value = &val;
 
 		/* Do we have the extended gem_create_ioctl? */
@@ -453,7 +503,7 @@ uint32_t __gem_create_stolen(int fd, uint64_t size)
 	create.handle = 0;
 	create.size = size;
 	create.flags = I915_CREATE_PLACEMENT_STOLEN;
-	ret = drmIoctl(fd, LOCAL_IOCTL_I915_GEM_CREATE, &create);
+	ret = igt_ioctl(fd, LOCAL_IOCTL_I915_GEM_CREATE, &create);
 
 	if (ret < 0)
 		return 0;
@@ -496,7 +546,7 @@ uint32_t __gem_create(int fd, int size)
 	memset(&create, 0, sizeof(create));
 	create.handle = 0;
 	create.size = size;
-	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_CREATE, &create);
+	ret = igt_ioctl(fd, DRM_IOCTL_I915_GEM_CREATE, &create);
 
 	if (ret < 0)
 		return 0;
@@ -539,8 +589,9 @@ uint32_t gem_create(int fd, uint64_t size)
 int __gem_execbuf(int fd, struct drm_i915_gem_execbuffer2 *execbuf)
 {
 	int err = 0;
-	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, execbuf))
+	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, execbuf))
 		err = -errno;
+	errno = 0;
 	return err;
 }
 
@@ -555,7 +606,6 @@ int __gem_execbuf(int fd, struct drm_i915_gem_execbuffer2 *execbuf)
 void gem_execbuf(int fd, struct drm_i915_gem_execbuffer2 *execbuf)
 {
 	igt_assert_eq(__gem_execbuf(fd, execbuf), 0);
-	errno = 0;
 }
 
 /**
@@ -577,7 +627,7 @@ void *__gem_mmap__gtt(int fd, uint32_t handle, uint64_t size, unsigned prot)
 
 	memset(&mmap_arg, 0, sizeof(mmap_arg));
 	mmap_arg.handle = handle;
-	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_MMAP_GTT, &mmap_arg))
+	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_MMAP_GTT, &mmap_arg))
 		return NULL;
 
 	ptr = mmap64(0, size, prot, MAP_SHARED, fd, mmap_arg.offset);
@@ -643,7 +693,7 @@ bool gem_mmap__has_wc(int fd)
 			arg.offset = 0;
 			arg.size = 4096;
 			arg.flags = I915_MMAP_WC;
-			has_wc = drmIoctl(fd, LOCAL_IOCTL_I915_GEM_MMAP_v2, &arg) == 0;
+			has_wc = igt_ioctl(fd, LOCAL_IOCTL_I915_GEM_MMAP_v2, &arg) == 0;
 			gem_close(fd, arg.handle);
 		}
 		errno = 0;
@@ -681,7 +731,7 @@ void *__gem_mmap__wc(int fd, uint32_t handle, uint64_t offset, uint64_t size, un
 	arg.offset = offset;
 	arg.size = size;
 	arg.flags = I915_MMAP_WC;
-	if (drmIoctl(fd, LOCAL_IOCTL_I915_GEM_MMAP_v2, &arg))
+	if (igt_ioctl(fd, LOCAL_IOCTL_I915_GEM_MMAP_v2, &arg))
 		return NULL;
 
 	errno = 0;
@@ -728,7 +778,7 @@ void *__gem_mmap__cpu(int fd, uint32_t handle, uint64_t offset, uint64_t size, u
 	mmap_arg.handle = handle;
 	mmap_arg.offset = offset;
 	mmap_arg.size = size;
-	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_MMAP, &mmap_arg))
+	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_MMAP, &mmap_arg))
 		return NULL;
 
 	errno = 0;
@@ -797,7 +847,7 @@ uint32_t gem_context_create(int fd)
 	struct drm_i915_gem_context_create create;
 
 	memset(&create, 0, sizeof(create));
-	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_CONTEXT_CREATE, &create)) {
+	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_CONTEXT_CREATE, &create)) {
 		int err = -errno;
 		igt_skip_on(err == -ENODEV || errno == -EINVAL);
 		igt_assert_eq(err, 0);
@@ -816,7 +866,7 @@ int __gem_context_destroy(int fd, uint32_t ctx_id)
 	memset(&destroy, 0, sizeof(destroy));
 	destroy.ctx_id = ctx_id;
 
-	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_CONTEXT_DESTROY, &destroy);
+	ret = igt_ioctl(fd, DRM_IOCTL_I915_GEM_CONTEXT_DESTROY, &destroy);
 	if (ret)
 		return -errno;
 	return 0;
@@ -840,6 +890,17 @@ void gem_context_destroy(int fd, uint32_t ctx_id)
 	do_ioctl(fd, DRM_IOCTL_I915_GEM_CONTEXT_DESTROY, &destroy);
 }
 
+int __gem_context_get_param(int fd, struct local_i915_gem_context_param *p)
+{
+#define LOCAL_I915_GEM_CONTEXT_GETPARAM       0x34
+#define LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM DRM_IOWR (DRM_COMMAND_BASE + LOCAL_I915_GEM_CONTEXT_GETPARAM, struct local_i915_gem_context_param)
+	if (igt_ioctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM, p))
+		return -errno;
+
+	errno = 0;
+	return 0;
+}
+
 /**
  * gem_context_get_param:
  * @fd: open i915 drm file descriptor
@@ -852,16 +913,15 @@ void gem_context_destroy(int fd, uint32_t ctx_id)
  */
 void gem_context_get_param(int fd, struct local_i915_gem_context_param *p)
 {
-#define LOCAL_I915_GEM_CONTEXT_GETPARAM       0x34
-#define LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM DRM_IOWR (DRM_COMMAND_BASE + LOCAL_I915_GEM_CONTEXT_GETPARAM, struct local_i915_gem_context_param)
-	do_ioctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM, p);
+	igt_assert(__gem_context_get_param(fd, p) == 0);
 }
 
+
 int __gem_context_set_param(int fd, struct local_i915_gem_context_param *p)
 {
 #define LOCAL_I915_GEM_CONTEXT_SETPARAM       0x35
 #define LOCAL_IOCTL_I915_GEM_CONTEXT_SETPARAM DRM_IOWR (DRM_COMMAND_BASE + LOCAL_I915_GEM_CONTEXT_SETPARAM, struct local_i915_gem_context_param)
-	if (drmIoctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_SETPARAM, p))
+	if (igt_ioctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_SETPARAM, p))
 		return -errno;
 
 	errno = 0;
@@ -899,12 +959,24 @@ void gem_context_require_param(int fd, uint64_t param)
 	p.value = 0;
 	p.size = 0;
 
-	igt_require(drmIoctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM, &p) == 0);
+	igt_require(igt_ioctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM, &p) == 0);
 }
 
-void gem_context_require_ban_period(int fd)
+void gem_context_require_bannable(int fd)
 {
 	static int has_ban_period = -1;
+	static int has_bannable = -1;
+
+	if (has_bannable < 0) {
+		struct local_i915_gem_context_param p;
+
+		p.context = 0;
+		p.param = LOCAL_CONTEXT_PARAM_BANNABLE;
+		p.value = 0;
+		p.size = 0;
+
+		has_bannable = igt_ioctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM, &p) == 0;
+	}
 
 	if (has_ban_period < 0) {
 		struct local_i915_gem_context_param p;
@@ -914,16 +986,15 @@ void gem_context_require_ban_period(int fd)
 		p.value = 0;
 		p.size = 0;
 
-		has_ban_period = drmIoctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM, &p) == 0;
+		has_ban_period = igt_ioctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM, &p) == 0;
 	}
 
-	igt_require(has_ban_period);
+	igt_require(has_ban_period || has_bannable);
 }
 
 int __gem_userptr(int fd, void *ptr, int size, int read_only, uint32_t flags, uint32_t *handle)
 {
 	struct local_i915_gem_userptr userptr;
-	int ret;
 
 	memset(&userptr, 0, sizeof(userptr));
 	userptr.user_ptr = (uintptr_t)ptr;
@@ -932,17 +1003,11 @@ int __gem_userptr(int fd, void *ptr, int size, int read_only, uint32_t flags, ui
 	if (read_only)
 		userptr.flags |= LOCAL_I915_USERPTR_READ_ONLY;
 
-	ret = drmIoctl(fd, LOCAL_IOCTL_I915_GEM_USERPTR, &userptr);
-	if (ret)
-		ret = errno;
-	igt_skip_on_f(ret == ENODEV &&
-			(flags & LOCAL_I915_USERPTR_UNSYNCHRONIZED) == 0 &&
-			!read_only,
-			"Skipping, synchronized mappings with no kernel CONFIG_MMU_NOTIFIER?");
-	if (ret == 0)
-		*handle = userptr.handle;
+	if (igt_ioctl(fd, LOCAL_IOCTL_I915_GEM_USERPTR, &userptr))
+		return -errno;
 
-	return ret;
+	*handle = userptr.handle;
+	return 0;
 }
 
 /**
@@ -1109,54 +1174,7 @@ bool gem_has_llc(int fd)
 	return has_llc;
 }
 
-/**
- * gem_get_num_rings:
- * @fd: open i915 drm file descriptor
- *
- * Feature test macro to query the number of available rings. This is useful in
- * test loops which need to step through all rings and similar logic.
- *
- * For more explicit tests of ring availability see gem_has_enable_ring() and
- * the ring specific versions like gem_has_bsd().
- *
- * Returns: The number of available rings.
- */
-int gem_get_num_rings(int fd)
-{
-	static int num_rings = -1;
-
-	if (num_rings < 0) {
-		num_rings = 1;	/* render ring is always available */
-
-		if (gem_has_bsd(fd))
-			num_rings++;
-		else
-			goto skip;
-
-		if (gem_has_blt(fd))
-			num_rings++;
-		else
-			goto skip;
-
-		if (gem_has_vebox(fd))
-			num_rings++;
-		else
-			goto skip;
-	}
-skip:
-	return num_rings;
-}
-
-/**
- * gem_has_enable_ring:
- * @fd: open i915 drm file descriptor
- * @param: ring flag bit as used in gem_execbuf()
- *
- * Feature test macro to query whether a specific ring is available.
- *
- * Returns: Whether the ring is available or not.
- */
-bool gem_has_enable_ring(int fd,int param)
+static bool has_param(int fd, int param)
 {
 	drm_i915_getparam_t gp;
 	int tmp = 0;
@@ -1165,7 +1183,7 @@ bool gem_has_enable_ring(int fd,int param)
 	gp.value = &tmp;
 	gp.param = param;
 
-	if (drmIoctl(fd, DRM_IOCTL_I915_GETPARAM, &gp))
+	if (igt_ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp))
 		return false;
 
 	errno = 0;
@@ -1176,8 +1194,7 @@ bool gem_has_enable_ring(int fd,int param)
  * gem_has_bsd:
  * @fd: open i915 drm file descriptor
  *
- * Feature test macro to query whether the BSD ring is available. This is simply
- * a specific version of gem_has_enable_ring() for the BSD ring.
+ * Feature test macro to query whether the BSD ring is available.
  *
  * Note that recent Bspec calls this the VCS ring for Video Command Submission.
  *
@@ -1187,7 +1204,7 @@ bool gem_has_bsd(int fd)
 {
 	static int has_bsd = -1;
 	if (has_bsd < 0)
-		has_bsd = gem_has_enable_ring(fd,I915_PARAM_HAS_BSD);
+		has_bsd = has_param(fd, I915_PARAM_HAS_BSD);
 	return has_bsd;
 }
 
@@ -1195,8 +1212,7 @@ bool gem_has_bsd(int fd)
  * gem_has_blt:
  * @fd: open i915 drm file descriptor
  *
- * Feature test macro to query whether the blitter ring is available. This is simply
- * a specific version of gem_has_enable_ring() for the blitter ring.
+ * Feature test macro to query whether the blitter ring is available.
  *
  * Note that recent Bspec calls this the BCS ring for Blitter Command Submission.
  *
@@ -1206,7 +1222,7 @@ bool gem_has_blt(int fd)
 {
 	static int has_blt = -1;
 	if (has_blt < 0)
-		has_blt =  gem_has_enable_ring(fd,I915_PARAM_HAS_BLT);
+		has_blt =  has_param(fd, I915_PARAM_HAS_BLT);
 	return has_blt;
 }
 
@@ -1215,8 +1231,7 @@ bool gem_has_blt(int fd)
  * gem_has_vebox:
  * @fd: open i915 drm file descriptor
  *
- * Feature test macro to query whether the vebox ring is available. This is simply
- * a specific version of gem_has_enable_ring() for the vebox ring.
+ * Feature test macro to query whether the vebox ring is available.
  *
  * Note that recent Bspec calls this the VECS ring for Video Enhancement Command
  * Submission.
@@ -1227,7 +1242,7 @@ bool gem_has_vebox(int fd)
 {
 	static int has_vebox = -1;
 	if (has_vebox < 0)
-		has_vebox =  gem_has_enable_ring(fd,LOCAL_I915_PARAM_HAS_VEBOX);
+		has_vebox =  has_param(fd, LOCAL_I915_PARAM_HAS_VEBOX);
 	return has_vebox;
 }
 
@@ -1236,8 +1251,7 @@ bool gem_has_vebox(int fd)
  * gem_has_bsd2:
  * @fd: open i915 drm file descriptor
  *
- * Feature test macro to query whether the BSD2 ring is available. This is simply
- * a specific version of gem_has_enable_ring() for the BSD2 ring.
+ * Feature test macro to query whether the BSD2 ring is available.
  *
  * Note that recent Bspec calls this the VCS ring for Video Command Submission.
  *
@@ -1247,9 +1261,55 @@ bool gem_has_bsd2(int fd)
 {
 	static int has_bsd2 = -1;
 	if (has_bsd2 < 0)
-		has_bsd2 = gem_has_enable_ring(fd,LOCAL_I915_PARAM_HAS_BSD2);
+		has_bsd2 = has_param(fd, LOCAL_I915_PARAM_HAS_BSD2);
 	return has_bsd2;
 }
+
+struct local_i915_gem_get_aperture {
+	__u64 aper_size;
+	__u64 aper_available_size;
+	__u64 version;
+	__u64 map_total_size;
+	__u64 stolen_total_size;
+};
+#define DRM_I915_GEM_GET_APERTURE	0x23
+#define LOCAL_IOCTL_I915_GEM_GET_APERTURE DRM_IOR  (DRM_COMMAND_BASE + DRM_I915_GEM_GET_APERTURE, struct local_i915_gem_get_aperture)
+/**
+ * gem_total_mappable_size:
+ * @fd: open i915 drm file descriptor
+ *
+ * Feature test macro to query the kernel for the total mappable size.
+ *
+ * Returns: Total mappable address space size.
+ */
+uint64_t gem_total_mappable_size(int fd)
+{
+	struct local_i915_gem_get_aperture aperture;
+
+	memset(&aperture, 0, sizeof(aperture));
+	do_ioctl(fd, LOCAL_IOCTL_I915_GEM_GET_APERTURE, &aperture);
+
+	return aperture.map_total_size;
+}
+
+/**
+ * gem_total_stolen_size:
+ * @fd: open i915 drm file descriptor
+ *
+ * Feature test macro to query the kernel for the total stolen size.
+ *
+ * Returns: Total stolen memory.
+ */
+uint64_t gem_total_stolen_size(int fd)
+{
+	struct local_i915_gem_get_aperture aperture;
+
+	memset(&aperture, 0, sizeof(aperture));
+	do_ioctl(fd, LOCAL_IOCTL_I915_GEM_GET_APERTURE, &aperture);
+
+	return aperture.stolen_total_size;
+}
+
 /**
  * gem_available_aperture_size:
  * @fd: open i915 drm file descriptor
@@ -1326,6 +1386,7 @@ uint64_t gem_mappable_aperture_size(void)
 
 /**
  * gem_global_aperture_size:
+ * @fd: open i915 drm file descriptor
  *
  * Feature test macro to query the kernel for the global gpu aperture size.
  * This is the area available for the kernel to perform address translations.
@@ -1381,60 +1442,70 @@ bool gem_has_softpin(int fd)
  */
 void gem_require_caching(int fd)
 {
-	struct local_drm_i915_gem_caching arg;
-	int ret;
+	uint32_t handle;
 
-	memset(&arg, 0, sizeof(arg));
-	arg.handle = gem_create(fd, 4096);
-	igt_assert(arg.handle != 0);
+	handle = gem_create(fd, 4096);
+	gem_set_caching(fd, handle, 0);
+	gem_close(fd, handle);
 
-	arg.caching = 0;
-	ret = ioctl(fd, LOCAL_DRM_IOCTL_I915_GEM_SET_CACHEING, &arg);
-	gem_close(fd, arg.handle);
-
-	igt_require(ret == 0);
 	errno = 0;
 }
 
-static int gem_has_ring(int fd, int ring)
+bool gem_has_ring(int fd, unsigned ring)
 {
-	uint32_t bbe = MI_BATCH_BUFFER_END;
 	struct drm_i915_gem_execbuffer2 execbuf;
 	struct drm_i915_gem_exec_object2 exec;
-	int ret;
+
+	/* silly ABI, the kernel thinks everyone who has BSD also has BSD2 */
+	if ((ring & ~(3<<13)) == I915_EXEC_BSD) {
+		if (ring & (3 << 13) && !gem_has_bsd2(fd))
+			return false;
+	}
 
 	memset(&exec, 0, sizeof(exec));
-	exec.handle = gem_create(fd, 4096);
-	gem_write(fd, exec.handle, 0, &bbe, sizeof(bbe));
-
 	memset(&execbuf, 0, sizeof(execbuf));
 	execbuf.buffers_ptr = (uintptr_t)&exec;
 	execbuf.buffer_count = 1;
 	execbuf.flags = ring;
-	ret = __gem_execbuf(fd, &execbuf);
-	gem_close(fd, exec.handle);
-
-	return ret == 0;
+	return __gem_execbuf(fd, &execbuf) == -ENOENT;
 }
 
 /**
  * gem_require_ring:
  * @fd: open i915 drm file descriptor
- * @ring_id: ring flag bit as used in gem_execbuf()
+ * @ring: ring flag bit as used in gem_execbuf()
  *
  * Feature test macro to query whether a specific ring is available.
- * In contrast to gem_has_enable_ring() this automagically skips if the ring
- * isn't available by calling igt_require().
+ * This automagically skips if the ring isn't available by
+ * calling igt_require().
  */
-void gem_require_ring(int fd, int ring_id)
+void gem_require_ring(int fd, unsigned ring)
 {
-	igt_require(gem_has_ring(fd, ring_id));
+	igt_require(gem_has_ring(fd, ring));
+}
 
-	/* silly ABI, the kernel thinks everyone who has BSD also has BSD2 */
-	if ((ring_id & ~(3<<13)) == I915_EXEC_BSD) {
-		if (ring_id & (3 << 13))
-			igt_require(gem_has_bsd2(fd));
-	}
+/**
+ * gem_has_mocs_registers:
+ * @fd: open i915 drm file descriptor
+ *
+ * Feature test macro to query whether the device has MOCS registers.
+ * These exist gen 9+.
+ */
+bool gem_has_mocs_registers(int fd)
+{
+	return intel_gen(intel_get_drm_devid(fd)) >= 9;
+}
+
+/**
+ * gem_require_mocs_registers:
+ * @fd: open i915 drm file descriptor
+ *
+ * Feature test macro to query whether the device has MOCS registers.
+ * These exist gen 9+.
+ */
+void gem_require_mocs_registers(int fd)
+{
+	igt_require(gem_has_mocs_registers(fd));
 }
 
 /* prime */
@@ -1483,7 +1554,7 @@ int prime_handle_to_fd_for_mmap(int fd, uint32_t handle)
 	args.flags = DRM_CLOEXEC | DRM_RDWR;
 	args.fd = -1;
 
-	if (drmIoctl(fd, DRM_IOCTL_PRIME_HANDLE_TO_FD, &args) != 0)
+	if (igt_ioctl(fd, DRM_IOCTL_PRIME_HANDLE_TO_FD, &args) != 0)
 		return -1;
 
 	return args.fd;
@@ -1538,6 +1609,9 @@ off_t prime_get_size(int dma_buf_fd)
 /**
  * prime_sync_start
  * @dma_buf_fd: dma-buf fd handle
+ * @write: read/write or read-only access
+ *
+ * Must be called before starting CPU mmap access to a dma-buf.
  */
 void prime_sync_start(int dma_buf_fd, bool write)
 {
@@ -1554,6 +1628,9 @@ void prime_sync_start(int dma_buf_fd, bool write)
 /**
  * prime_sync_end
  * @dma_buf_fd: dma-buf fd handle
+ * @write: read/write or read-only access
+ *
+ * Must be called after finishing CPU mmap access to a dma-buf.
  */
 void prime_sync_end(int dma_buf_fd, bool write)
 {
@@ -1609,7 +1686,7 @@ int __kms_addfb(int fd, uint32_t handle, uint32_t width, uint32_t height,
 	f.pitches[0] = stride;
 	f.modifier[0] = modifier;
 
-	ret = drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f);
+	ret = igt_ioctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f);
 
 	*buf_id = f.fb_id;
 
