@@ -22,6 +22,7 @@
  */
 
 #include "igt.h"
+#include "igt_rand.h"
 
 #define LOCAL_EXEC_NO_RELOC (1<<11)
 
@@ -65,7 +66,6 @@ static uint32_t busy_blt(int fd)
 	struct drm_i915_gem_execbuffer2 execbuf;
 	struct drm_i915_gem_exec_object2 object[2];
 	struct drm_i915_gem_relocation_entry reloc[200], *r;
-	uint32_t read, write;
 	uint32_t *map;
 	int factor = 100;
 	int i = 0;
@@ -126,10 +126,7 @@ static uint32_t busy_blt(int fd)
 	if (gen >= 6)
 		execbuf.flags = I915_EXEC_BLT;
 	gem_execbuf(fd, &execbuf);
-
-	__gem_busy(fd, object[0].handle, &read, &write);
-	igt_assert_eq(read, 1 << write);
-	igt_assert_eq(write, gen >= 6 ? I915_EXEC_BLT : I915_EXEC_RENDER);
+	igt_assert(gem_bo_busy(fd, object[0].handle));
 
 	igt_debug("Created busy handle %d\n", object[0].handle);
 	gem_close(fd, object[1].handle);
@@ -223,6 +220,7 @@ static void semaphore(int fd, unsigned ring, uint32_t flags)
 }
 
 #define PARALLEL 1
+#define HANG 2
 static void one(int fd, unsigned ring, uint32_t flags, unsigned test_flags)
 {
 	const int gen = intel_gen(intel_get_drm_devid(fd));
@@ -235,7 +233,7 @@ static void one(int fd, unsigned ring, uint32_t flags, unsigned test_flags)
 	uint32_t read[2], write[2];
 	struct timespec tv;
 	uint32_t *batch, *bbe;
-	int i, count;
+	int i, count, timeout;
 
 	memset(&execbuf, 0, sizeof(execbuf));
 	execbuf.buffers_ptr = (uintptr_t)obj;
@@ -329,8 +327,12 @@ static void one(int fd, unsigned ring, uint32_t flags, unsigned test_flags)
 		}
 	}
 
-	*bbe = MI_BATCH_BUFFER_END;
-	__sync_synchronize();
+	timeout = 120;
+	if ((test_flags & HANG) == 0) {
+		*bbe = MI_BATCH_BUFFER_END;
+		__sync_synchronize();
+		timeout = 1;
+	}
 
 	igt_assert_eq(write[SCRATCH], ring);
 	igt_assert_eq_u32(read[SCRATCH], 1 << ring);
@@ -341,7 +343,7 @@ static void one(int fd, unsigned ring, uint32_t flags, unsigned test_flags)
 	/* Calling busy in a loop should be enough to flush the rendering */
 	memset(&tv, 0, sizeof(tv));
 	while (gem_busy(fd, obj[BATCH].handle))
-		igt_assert(igt_seconds_elapsed(&tv) < 10);
+		igt_assert(igt_seconds_elapsed(&tv) < timeout);
 	igt_assert(!gem_busy(fd, obj[SCRATCH].handle));
 
 	munmap(batch, size);
@@ -354,6 +356,93 @@ static void one(int fd, unsigned ring, uint32_t flags, unsigned test_flags)
 	gem_close(fd, obj[SCRATCH].handle);
 }
 
+static void xchg_u32(void *array, unsigned i, unsigned j)
+{
+	uint32_t *u32 = array;
+	uint32_t tmp = u32[i];
+	u32[i] = u32[j];
+	u32[j] = tmp;
+}
+
+static void close_race(int fd)
+{
+#define N_HANDLES 4096
+	const int ncpus = sysconf(_SC_NPROCESSORS_ONLN);
+	uint32_t *handles;
+	unsigned long *control;
+	unsigned long count = 0;
+	int i;
+
+	intel_require_memory(N_HANDLES, 4096, CHECK_RAM);
+
+	/* One thread spawning work and randomly closing fd.
+	 * One background thread per cpu checking busyness.
+	 */
+
+	control = mmap(NULL, 4096, PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
+	igt_assert(control != MAP_FAILED);
+
+	handles = mmap(NULL, N_HANDLES*sizeof(*handles),
+		   PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
+	igt_assert(handles != MAP_FAILED);
+
+	for (i = 0; i < N_HANDLES; i++) {
+		handles[i] = gem_create(fd, 4096);
+	}
+
+	igt_fork(child, ncpus) {
+		struct drm_i915_gem_busy busy;
+		uint32_t indirection[N_HANDLES];
+
+		for (i = 0; i < N_HANDLES; i++)
+			indirection[i] = i;
+
+		hars_petruska_f54_1_random_perturb(child);
+
+		memset(&busy, 0, sizeof(busy));
+		do {
+			igt_permute_array(indirection, N_HANDLES, xchg_u32);
+			__sync_synchronize();
+			for (i = 0; i < N_HANDLES; i++) {
+				busy.handle = indirection[handles[i]];
+				/* Check that the busy computation doesn't
+				 * explode in the face of random gem_close().
+				 */
+				drmIoctl(fd, DRM_IOCTL_I915_GEM_BUSY, &busy);
+			}
+			count++;
+		} while(*(volatile long *)control == 0);
+
+		igt_debug("child[%d]: count = %lu\n", child, count);
+		control[child] = count;
+	}
+
+	igt_until_timeout(20) {
+		int j = rand() % N_HANDLES;
+
+		gem_close(fd, handles[j]);
+		__sync_synchronize();
+		handles[j] = busy_blt(fd);
+
+		count++;
+	}
+	control[0] = 1;
+	igt_waitchildren();
+
+	for (i = 0; i < ncpus; i++)
+		control[ncpus + 1] += control[i + 1];
+	igt_info("Total execs %lu, busy-ioctls %lu\n",
+		 count, control[ncpus + 1] * N_HANDLES);
+
+	for (i = 0; i < N_HANDLES; i++)
+		gem_close(fd, handles[i]);
+
+	munmap(handles, N_HANDLES * sizeof(*handles));
+	munmap(control, 4096);
+
+	gem_quiescent_gpu(fd);
+}
+
 static bool has_semaphores(int fd)
 {
 	struct drm_i915_getparam gp;
@@ -369,6 +458,141 @@ static bool has_semaphores(int fd)
 	return val > 0;
 }
 
+static bool has_extended_busy_ioctl(int fd)
+{
+	const int gen = intel_gen(intel_get_drm_devid(fd));
+	struct drm_i915_gem_exec_object2 obj;
+	struct drm_i915_gem_relocation_entry reloc;
+	struct drm_i915_gem_execbuffer2 execbuf;
+	uint32_t read, write;
+	uint32_t *batch;
+	int i;
+
+	memset(&execbuf, 0, sizeof(execbuf));
+	execbuf.buffers_ptr = (uintptr_t)&obj;
+	execbuf.buffer_count = 1;
+
+	memset(&obj, 0, sizeof(obj));
+	obj.handle = gem_create(fd, 4096);
+
+	obj.relocs_ptr = (uintptr_t)&reloc;
+	obj.relocation_count = 1;
+	memset(&reloc, 0, sizeof(reloc));
+
+	batch = gem_mmap__gtt(fd, obj.handle, 4096, PROT_WRITE);
+	gem_set_domain(fd, obj.handle,
+			I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
+
+	reloc.target_handle = obj.handle; /* recurse */
+	reloc.presumed_offset = 0;
+	reloc.offset = sizeof(uint32_t);
+	reloc.delta = 0;
+	reloc.read_domains = I915_GEM_DOMAIN_COMMAND;
+	reloc.write_domain = 0;
+
+	i = 0;
+	batch[i] = MI_BATCH_BUFFER_START;
+	if (gen >= 8) {
+		batch[i] |= 1 << 8 | 1;
+		batch[++i] = 0;
+		batch[++i] = 0;
+	} else if (gen >= 6) {
+		batch[i] |= 1 << 8;
+		batch[++i] = 0;
+	} else {
+		batch[i] |= 2 << 6;
+		batch[++i] = 0;
+		if (gen < 4) {
+			batch[i] |= 1;
+			reloc.delta = 1;
+		}
+	}
+	i++;
+
+	gem_execbuf(fd, &execbuf);
+	__gem_busy(fd, obj.handle, &read, &write);
+
+	*batch = MI_BATCH_BUFFER_END;
+	__sync_synchronize();
+	munmap(batch, 4096);
+
+	gem_close(fd, obj.handle);
+
+	return read != 0;
+}
+
+static void basic(int fd, unsigned ring, unsigned flags)
+{
+	const int gen = intel_gen(intel_get_drm_devid(fd));
+	struct drm_i915_gem_exec_object2 obj;
+	struct drm_i915_gem_relocation_entry reloc;
+	struct drm_i915_gem_execbuffer2 execbuf;
+	struct timespec tv;
+	uint32_t *batch;
+	int i, timeout;
+	bool busy;
+
+	memset(&execbuf, 0, sizeof(execbuf));
+	execbuf.buffers_ptr = (uintptr_t)&obj;
+	execbuf.buffer_count = 1;
+	execbuf.flags = ring;
+
+	memset(&obj, 0, sizeof(obj));
+	obj.handle = gem_create(fd, 4096);
+
+	obj.relocs_ptr = (uintptr_t)&reloc;
+	obj.relocation_count = 1;
+	memset(&reloc, 0, sizeof(reloc));
+
+	batch = gem_mmap__gtt(fd, obj.handle, 4096, PROT_WRITE);
+	gem_set_domain(fd, obj.handle,
+			I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
+
+	reloc.target_handle = obj.handle; /* recurse */
+	reloc.presumed_offset = 0;
+	reloc.offset = sizeof(uint32_t);
+	reloc.delta = 0;
+	reloc.read_domains = I915_GEM_DOMAIN_COMMAND;
+	reloc.write_domain = 0;
+
+	i = 0;
+	batch[i] = MI_BATCH_BUFFER_START;
+	if (gen >= 8) {
+		batch[i] |= 1 << 8 | 1;
+		batch[++i] = 0;
+		batch[++i] = 0;
+	} else if (gen >= 6) {
+		batch[i] |= 1 << 8;
+		batch[++i] = 0;
+	} else {
+		batch[i] |= 2 << 6;
+		batch[++i] = 0;
+		if (gen < 4) {
+			batch[i] |= 1;
+			reloc.delta = 1;
+		}
+	}
+	i++;
+
+	gem_execbuf(fd, &execbuf);
+	busy = gem_bo_busy(fd, obj.handle);
+
+	timeout = 120;
+	if ((flags & HANG) == 0) {
+		*batch = MI_BATCH_BUFFER_END;
+		__sync_synchronize();
+		timeout = 1;
+	}
+	munmap(batch, 4096);
+
+	igt_assert(busy);
+	memset(&tv, 0, sizeof(tv));
+	while (gem_bo_busy(fd, obj.handle))
+		igt_assert(igt_seconds_elapsed(&tv) < timeout);
+
+	gem_close(fd, obj.handle);
+}
+
 igt_main
 {
 	const struct intel_execution_engine *e;
@@ -376,62 +600,142 @@ igt_main
 
 	igt_skip_on_simulation();
 
-	igt_fixture
+	igt_fixture {
 		fd = drm_open_driver_master(DRIVER_INTEL);
+	}
 
 	igt_subtest_group {
-		int gen = 0;
+		igt_fixture {
+			igt_fork_hang_detector(fd);
+		}
+
+		for (e = intel_execution_engines; e->name; e++) {
+			igt_subtest_group {
+				igt_subtest_f("%sbusy-%s",
+					      e->exec_id == 0 ? "basic-" : "",
+					      e->name) {
+					igt_require(gem_has_ring(fd, e->exec_id | e->flags));
+					gem_quiescent_gpu(fd);
+					basic(fd, e->exec_id | e->flags, 0);
+				}
+			}
+		}
+
+		igt_subtest_group {
+			int gen = 0;
+
+			igt_fixture {
+				igt_require(has_extended_busy_ioctl(fd));
+				gem_require_mmap_wc(fd);
+				gen = intel_gen(intel_get_drm_devid(fd));
+			}
+
+			for (e = intel_execution_engines; e->name; e++) {
+				/* default exec-id is purely symbolic */
+				if (e->exec_id == 0)
+					continue;
+
+				igt_subtest_f("extended-%s", e->name) {
+					gem_require_ring(fd, e->exec_id | e->flags);
+					igt_skip_on_f(gen == 6 &&
+						      e->exec_id == I915_EXEC_BSD,
+						      "MI_STORE_DATA broken on gen6 bsd\n");
+					gem_quiescent_gpu(fd);
+					one(fd, e->exec_id, e->flags, 0);
+					gem_quiescent_gpu(fd);
+				}
+			}
+
+			for (e = intel_execution_engines; e->name; e++) {
+				/* default exec-id is purely symbolic */
+				if (e->exec_id == 0)
+					continue;
+
+				igt_subtest_f("extended-parallel-%s", e->name) {
+					gem_require_ring(fd, e->exec_id | e->flags);
+					igt_skip_on_f(gen == 6 &&
+						      e->exec_id == I915_EXEC_BSD,
+						      "MI_STORE_DATA broken on gen6 bsd\n");
+					gem_quiescent_gpu(fd);
+					one(fd, e->exec_id, e->flags, PARALLEL);
+					gem_quiescent_gpu(fd);
+				}
+			}
+		}
+
+		igt_subtest_group {
+			igt_fixture {
+				igt_require(has_extended_busy_ioctl(fd));
+				igt_require(has_semaphores(fd));
+			}
+
+			for (e = intel_execution_engines; e->name; e++) {
+				/* default exec-id is purely symbolic */
+				if (e->exec_id == 0)
+					continue;
+
+				igt_subtest_f("extended-semaphore-%s", e->name)
+					semaphore(fd, e->exec_id, e->flags);
+			}
+		}
+
+		igt_subtest("close-race")
+			close_race(fd);
 
 		igt_fixture {
-			gem_require_mmap_wc(fd);
-			gen = intel_gen(intel_get_drm_devid(fd));
-		}
-
-		for (e = intel_execution_engines; e->name; e++) {
-			/* default exec-id is purely symbolic */
-			if (e->exec_id == 0)
-				continue;
-
-			igt_subtest_f("basic-%s", e->name) {
-				gem_require_ring(fd, e->exec_id | e->flags);
-				igt_skip_on_f(gen == 6 &&
-					      e->exec_id == I915_EXEC_BSD,
-					      "MI_STORE_DATA broken on gen6 bsd\n");
-				gem_quiescent_gpu(fd);
-				one(fd, e->exec_id, e->flags, 0);
-			}
-		}
-
-		for (e = intel_execution_engines; e->name; e++) {
-			/* default exec-id is purely symbolic */
-			if (e->exec_id == 0)
-				continue;
-
-			igt_subtest_f("basic-parallel-%s", e->name) {
-				gem_require_ring(fd, e->exec_id | e->flags);
-				igt_skip_on_f(gen == 6 &&
-					      e->exec_id == I915_EXEC_BSD,
-					      "MI_STORE_DATA broken on gen6 bsd\n");
-				gem_quiescent_gpu(fd);
-				one(fd, e->exec_id, e->flags, PARALLEL);
-			}
+			igt_stop_hang_detector();
 		}
 	}
 
 	igt_subtest_group {
-		igt_fixture
-			igt_require(has_semaphores(fd));
+		igt_hang_t hang;
+
+		igt_fixture {
+			hang = igt_allow_hang(fd, 0, 0);
+		}
 
 		for (e = intel_execution_engines; e->name; e++) {
-			/* default exec-id is purely symbolic */
-			if (e->exec_id == 0)
-				continue;
+			igt_subtest_f("%shang-%s",
+				      e->exec_id == 0 ? "basic-" : "",
+				      e->name) {
+				igt_require(gem_has_ring(fd, e->exec_id | e->flags));
+				gem_quiescent_gpu(fd);
+				basic(fd, e->exec_id | e->flags, HANG);
+			}
+		}
 
-			igt_subtest_f("semaphore-%s", e->name)
-				semaphore(fd, e->exec_id, e->flags);
+		igt_subtest_group {
+			int gen = 0;
+
+			igt_fixture {
+				igt_require(has_extended_busy_ioctl(fd));
+				gem_require_mmap_wc(fd);
+				gen = intel_gen(intel_get_drm_devid(fd));
+			}
+
+			for (e = intel_execution_engines; e->name; e++) {
+				/* default exec-id is purely symbolic */
+				if (e->exec_id == 0)
+					continue;
+
+				igt_subtest_f("extended-hang-%s", e->name) {
+					gem_require_ring(fd, e->exec_id | e->flags);
+					igt_skip_on_f(gen == 6 &&
+						      e->exec_id == I915_EXEC_BSD,
+						      "MI_STORE_DATA broken on gen6 bsd\n");
+					gem_quiescent_gpu(fd);
+					one(fd, e->exec_id, e->flags, HANG);
+					gem_quiescent_gpu(fd);
+				}
+			}
+		}
+
+		igt_fixture {
+			igt_disallow_hang(fd, hang);
 		}
 	}
 
-	igt_fixture
+	igt_fixture {
 		close(fd);
+	}
 }
