@@ -387,12 +387,12 @@ static void
 dmabuf_set_bo(struct buffers *b, drm_intel_bo *bo, uint32_t val)
 {
 	struct dmabuf *dmabuf = bo->virtual;
-	uint32_t *v;
-	int size;
+	uint32_t *v = dmabuf->map;
+	int y;
 
 	prime_sync_start(dmabuf->fd, true);
-	for (v = dmabuf->map, size = b->npixels; size--; v++)
-		*v = val;
+	for (y = 0; y < b->height; y++)
+		v[pixel(y, b->width)] = val;
 	prime_sync_end(dmabuf->fd, true);
 }
 
@@ -400,12 +400,12 @@ static void
 dmabuf_cmp_bo(struct buffers *b, drm_intel_bo *bo, uint32_t val)
 {
 	struct dmabuf *dmabuf = bo->virtual;
-	uint32_t *v;
-	int size;
+	uint32_t *v = dmabuf->map;
+	int y;
 
 	prime_sync_start(dmabuf->fd, false);
-	for (v = dmabuf->map, size = b->npixels; size--; v++)
-		igt_assert_eq_u32(*v, val);
+	for (y = 0; y < b->height; y++)
+		igt_assert_eq_u32(v[pixel(y, b->width)], val);
 	prime_sync_end(dmabuf->fd, false);
 }
 
@@ -485,11 +485,10 @@ static void
 gtt_set_bo(struct buffers *b, drm_intel_bo *bo, uint32_t val)
 {
 	uint32_t *vaddr = bo->virtual;
-	int size = b->npixels;
 
 	drm_intel_gem_bo_start_gtt_access(bo, true);
-	while (size--)
-		*vaddr++ = val;
+	for (int y = 0; y < b->height; y++)
+		vaddr[pixel(y, b->width)] = val;
 }
 
 static void
@@ -820,16 +819,6 @@ static void buffers_create(struct buffers *b)
 
 static void buffers_reset(struct buffers *b, bool enable_reuse)
 {
-	buffers_destroy(b);
-
-	igt_assert(b->count == 0);
-	igt_assert(b->tmp);
-	igt_assert(b->src);
-	igt_assert(b->dst);
-
-	intel_batchbuffer_free(b->batch);
-	drm_intel_bufmgr_destroy(b->bufmgr);
-
 	b->bufmgr = drm_intel_bufmgr_gem_init(fd, 4096);
 	igt_assert(b->bufmgr);
 
@@ -856,7 +845,7 @@ static void buffers_fini(struct buffers *b)
 }
 
 typedef void (*do_copy)(struct buffers *b, drm_intel_bo *dst, drm_intel_bo *src);
-typedef struct igt_hang_ring (*do_hang)(void);
+typedef igt_hang_t (*do_hang)(void);
 
 static void render_copy_bo(struct buffers *b, drm_intel_bo *dst, drm_intel_bo *src)
 {
@@ -940,27 +929,27 @@ static void wc_copy_bo(struct buffers *b, drm_intel_bo *dst, drm_intel_bo *src)
 	munmap(s, size);
 }
 
-static struct igt_hang_ring no_hang(void)
+static igt_hang_t no_hang(void)
 {
-	return (struct igt_hang_ring){0, 0};
+	return (igt_hang_t){0, 0};
 }
 
-static struct igt_hang_ring bcs_hang(void)
+static igt_hang_t bcs_hang(void)
 {
 	return igt_hang_ring(fd, I915_EXEC_BLT);
 }
 
-static struct igt_hang_ring rcs_hang(void)
+static igt_hang_t rcs_hang(void)
 {
 	return igt_hang_ring(fd, I915_EXEC_RENDER);
 }
 
-static struct igt_hang_ring all_hang(void)
+static igt_hang_t all_hang(void)
 {
 	uint32_t bbe = MI_BATCH_BUFFER_END;
 	struct drm_i915_gem_execbuffer2 execbuf;
 	struct drm_i915_gem_exec_object2 obj;
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 	unsigned engine;
 
 	memset(&obj, 0, sizeof(obj));
@@ -992,7 +981,7 @@ static void do_basic0(struct buffers *buffers,
 
 	buffers->mode->set_bo(buffers, buffers->src[0], 0xdeadbeef);
 	for (int i = 0; i < buffers->count; i++) {
-		struct igt_hang_ring hang = do_hang_func();
+		igt_hang_t hang = do_hang_func();
 
 		do_copy_func(buffers, buffers->dst[i], buffers->src[0]);
 		buffers->mode->cmp_bo(buffers, buffers->dst[i], 0xdeadbeef);
@@ -1008,7 +997,7 @@ static void do_basic1(struct buffers *buffers,
 	gem_quiescent_gpu(fd);
 
 	for (int i = 0; i < buffers->count; i++) {
-		struct igt_hang_ring hang = do_hang_func();
+		igt_hang_t hang = do_hang_func();
 
 		buffers->mode->set_bo(buffers, buffers->src[i], i);
 		buffers->mode->set_bo(buffers, buffers->dst[i], ~i);
@@ -1025,7 +1014,7 @@ static void do_basicN(struct buffers *buffers,
 		      do_copy do_copy_func,
 		      do_hang do_hang_func)
 {
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 
 	gem_quiescent_gpu(fd);
 
@@ -1051,7 +1040,7 @@ static void do_overwrite_source(struct buffers *buffers,
 				do_copy do_copy_func,
 				do_hang do_hang_func)
 {
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 	int i;
 
 	gem_quiescent_gpu(fd);
@@ -1075,7 +1064,7 @@ static void do_overwrite_source_read(struct buffers *buffers,
 				     int do_rcs)
 {
 	const int half = buffers->count/2;
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 	int i;
 
 	gem_quiescent_gpu(fd);
@@ -1119,7 +1108,7 @@ static void do_overwrite_source__rev(struct buffers *buffers,
 				     do_copy do_copy_func,
 				     do_hang do_hang_func)
 {
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 	int i;
 
 	gem_quiescent_gpu(fd);
@@ -1141,7 +1130,7 @@ static void do_overwrite_source__one(struct buffers *buffers,
 				     do_copy do_copy_func,
 				     do_hang do_hang_func)
 {
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 
 	gem_quiescent_gpu(fd);
 	buffers->mode->set_bo(buffers, buffers->src[0], 0);
@@ -1159,7 +1148,7 @@ static void do_intermix(struct buffers *buffers,
 			int do_rcs)
 {
 	const int half = buffers->count/2;
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 	int i;
 
 	gem_quiescent_gpu(fd);
@@ -1213,7 +1202,7 @@ static void do_early_read(struct buffers *buffers,
 			  do_copy do_copy_func,
 			  do_hang do_hang_func)
 {
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 	int i;
 
 	gem_quiescent_gpu(fd);
@@ -1231,7 +1220,7 @@ static void do_read_read_bcs(struct buffers *buffers,
 			     do_copy do_copy_func,
 			     do_hang do_hang_func)
 {
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 	int i;
 
 	gem_quiescent_gpu(fd);
@@ -1252,7 +1241,7 @@ static void do_write_read_bcs(struct buffers *buffers,
 			      do_copy do_copy_func,
 			      do_hang do_hang_func)
 {
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 	int i;
 
 	gem_quiescent_gpu(fd);
@@ -1272,7 +1261,7 @@ static void do_read_read_rcs(struct buffers *buffers,
 			     do_copy do_copy_func,
 			     do_hang do_hang_func)
 {
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 	int i;
 
 	gem_quiescent_gpu(fd);
@@ -1293,7 +1282,7 @@ static void do_write_read_rcs(struct buffers *buffers,
 			      do_copy do_copy_func,
 			      do_hang do_hang_func)
 {
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 	int i;
 
 	gem_quiescent_gpu(fd);
@@ -1313,7 +1302,7 @@ static void do_gpu_read_after_write(struct buffers *buffers,
 				    do_copy do_copy_func,
 				    do_hang do_hang_func)
 {
-	struct igt_hang_ring hang;
+	igt_hang_t hang;
 	int i;
 
 	gem_quiescent_gpu(fd);
@@ -1384,7 +1373,9 @@ static void __run_forked(struct buffers *buffers,
 
 {
 	/* purge the libdrm caches before cloing the process */
-	buffers_reset(buffers, true);
+	buffers_destroy(buffers);
+	intel_batchbuffer_free(buffers->batch);
+	drm_intel_bufmgr_destroy(buffers->bufmgr);
 
 	igt_fork(child, num_children) {
 		int num_buffers;
@@ -1409,6 +1400,8 @@ static void __run_forked(struct buffers *buffers,
 	}
 	igt_waitchildren();
 	igt_assert_eq(intel_detect_and_clear_missed_interrupts(fd), 0);
+
+	buffers_reset(buffers, true);
 }
 
 static void run_forked(struct buffers *buffers,
