@@ -201,7 +201,7 @@ igt_main
 		 * the last context is leaked at every reset.
 		 */
 		for (i = 0; i < 20; i++) {
-			igt_hang_ring_t hang = igt_hang_ring(fd, I915_EXEC_RENDER);
+			igt_hang_t hang = igt_hang_ring(fd, I915_EXEC_RENDER);
 			igt_assert(exec(fd, handle, I915_EXEC_RENDER, 0) == 0);
 			igt_assert(exec(fd, handle, I915_EXEC_RENDER, ctx_id) == 0);
 			igt_post_hang_ring(fd, hang);
