@@ -57,6 +57,25 @@ static bool has_execlists(int fd)
 	return enabled;
 }
 
+static unsigned context_size(int fd)
+{
+	const int gen = intel_gen(intel_get_drm_devid(fd));
+
+	switch (gen) {
+	case 0:
+	case 1:
+	case 2:
+	case 3:
+	case 4:
+	case 5:
+	case 6:
+	case 7: return 18 << 12;
+	case 8: return 20 << 12;
+	case 9: return 22 << 12;
+	default: return 32 << 12;
+	}
+}
+
 static unsigned get_num_contexts(int fd, int num_engines)
 {
 	uint64_t ggtt_size;
@@ -66,16 +85,17 @@ static unsigned get_num_contexts(int fd, int num_engines)
 	/* Compute the number of contexts we can allocate to fill the GGTT */
 	ggtt_size = gem_global_aperture_size(fd);
 
-	size = 64 << 10; /* Most gen require at least 64k for ctx */
+	size = context_size(fd);
 	if (has_execlists(fd)) {
-		size *= 2; /* ringbuffer as well */
+		size += 4 << 12; /* ringbuffer as well */
 		if (num_engines) /* one per engine with execlists */
 			size *= num_engines;
 	}
 
 	count = 3 * (ggtt_size / size) / 2;
-	igt_info("Creating %lld contexts (assuming of size %lld)\n",
-		 (long long)count, (long long)size);
+	igt_info("Creating %lld contexts (assuming of size %lld%s)\n",
+		 (long long)count, (long long)size,
+		 has_execlists(fd) ? " with execlists" : "");
 
 	intel_require_memory(count, size, CHECK_RAM | CHECK_SWAP);
 	return count;
@@ -342,11 +362,13 @@ static void *thread(void *data)
 	ctx = malloc(t->num_ctx * sizeof(uint32_t));
 	igt_assert(ctx);
 	memcpy(ctx, t->all_ctx, t->num_ctx * sizeof(uint32_t));
-	igt_permute_array(ctx, t->num_ctx, xchg_int);
 
-	for (unsigned n = 0; n < t->num_ctx; n++) {
-		execbuf.rsvd1 = ctx[n];
-		gem_execbuf(t->fd, &execbuf);
+	igt_until_timeout(150) {
+		igt_permute_array(ctx, t->num_ctx, xchg_int);
+		for (unsigned n = 0; n < t->num_ctx; n++) {
+			execbuf.rsvd1 = ctx[n];
+			gem_execbuf(t->fd, &execbuf);
+		}
 	}
 
 	free(ctx);
