@@ -26,7 +26,6 @@
  */
 
 #define _GNU_SOURCE
-#include "igt.h"
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -39,10 +38,15 @@
 #include <sys/ioctl.h>
 #include "drm.h"
 
+#include "igt.h"
+#include "igt_x86.h"
+
 #ifndef PAGE_SIZE
 #define PAGE_SIZE 4096
 #endif
 
+#define abs(x) ((x) >= 0 ? (x) : -(x))
+
 static int OBJECT_SIZE = 16*1024*1024;
 
 static void
@@ -321,6 +325,10 @@ test_coherency(int fd)
 	cpu = gem_mmap__cpu(fd, handle, 0, OBJECT_SIZE, PROT_READ | PROT_WRITE);
 	set_domain_gtt(fd, handle);
 
+	/* On byt/bsw/bxt this detects an interesting behaviour where the
+	 * CPU cannot flush the iobar and so the read may bypass the write.
+	 * https://bugs.freedesktop.org/show_bug.cgi?id=94314
+	 */
 	for (i = 0; i < OBJECT_SIZE / 64; i++) {
 		int x = 16*i + (i%16);
 		gtt[x] = i;
@@ -335,6 +343,16 @@ test_coherency(int fd)
 
 static int min_tile_width(uint32_t devid, int tiling)
 {
+	if (tiling < 0) {
+		tiling = -tiling;
+
+		if (intel_gen(devid) >= 4)
+			return 4096 - min_tile_width(devid, tiling);
+		else
+			return 1024;
+
+	}
+
 	if (intel_gen(devid) == 2)
 		return 128;
 	else if (tiling == I915_TILING_X)
@@ -347,6 +365,15 @@ static int min_tile_width(uint32_t devid, int tiling)
 
 static int max_tile_width(uint32_t devid, int tiling)
 {
+	if (tiling < 0) {
+		tiling = -tiling;
+
+		if (intel_gen(devid) >= 4)
+			return 4096 + min_tile_width(devid, tiling);
+		else
+			return 2048;
+	}
+
 	if (intel_gen(devid) >= 7)
 		return 256 << 10;
 	else if (intel_gen(devid) >= 4)
@@ -439,13 +466,52 @@ test_huge_bo(int fd, int huge, int tiling)
 	munmap(linear_pattern, PAGE_SIZE);
 }
 
-static void
-test_huge_copy(int fd, int huge, int tiling_a, int tiling_b)
+#if defined(__x86_64__) && !defined(__clang__)
+#define MOVNT 512
+
+#pragma GCC push_options
+#pragma GCC target("sse4.1")
+
+#include <smmintrin.h>
+__attribute__((noinline))
+static void copy_wc_page(void *dst, void *src)
 {
-	uint32_t devid = intel_get_drm_devid(fd);
+	if (igt_x86_features() & SSE4_1) {
+		__m128i *S = (__m128i *)src;
+		__m128i *D = (__m128i *)dst;
+
+		for (int i = 0; i < PAGE_SIZE/64; i++) {
+			__m128i tmp[4];
+
+			tmp[0] = _mm_stream_load_si128(S++);
+			tmp[1] = _mm_stream_load_si128(S++);
+			tmp[2] = _mm_stream_load_si128(S++);
+			tmp[3] = _mm_stream_load_si128(S++);
+
+			_mm_store_si128(D++, tmp[0]);
+			_mm_store_si128(D++, tmp[1]);
+			_mm_store_si128(D++, tmp[2]);
+			_mm_store_si128(D++, tmp[3]);
+		}
+	} else
+		memcpy(dst, src, PAGE_SIZE);
+}
+
+#pragma GCC pop_options
+
+#else
+static void copy_wc_page(void *dst, const void *src)
+{
+	memcpy(dst, src, PAGE_SIZE);
+}
+#endif
+
+static void
+test_huge_copy(int fd, int huge, int tiling_a, int tiling_b, int ncpus)
+{
+	const uint32_t devid = intel_get_drm_devid(fd);
 	uint64_t huge_object_size, i;
-	uint32_t bo, *pattern_a, *pattern_b;
-	char *a, *b;
+	unsigned mode = CHECK_RAM;
 
 	switch (huge) {
 	case -2:
@@ -457,72 +523,92 @@ test_huge_copy(int fd, int huge, int tiling_a, int tiling_b)
 	case 0:
 		huge_object_size = gem_mappable_aperture_size() + PAGE_SIZE;
 		break;
-	default:
+	case 1:
 		huge_object_size = gem_global_aperture_size(fd) + PAGE_SIZE;
 		break;
+	default:
+		huge_object_size = (intel_get_total_ram_mb() << 19) + PAGE_SIZE;
+		mode |= CHECK_SWAP;
+		break;
 	}
-	intel_require_memory(2, huge_object_size, CHECK_RAM);
+	intel_require_memory(2*ncpus, huge_object_size, mode);
 
-	pattern_a = malloc(PAGE_SIZE);
-	for (i = 0; i < PAGE_SIZE/4; i++)
-		pattern_a[i] = i;
+	igt_fork(child, ncpus) {
+		uint32_t bo;
+		char *a, *b;
 
-	pattern_b = malloc(PAGE_SIZE);
-	for (i = 0; i < PAGE_SIZE/4; i++)
-		pattern_b[i] = ~i;
+		bo = gem_create(fd, huge_object_size);
+		if (tiling_a)
+			igt_require(__gem_set_tiling(fd, bo, abs(tiling_a), min_tile_width(devid, tiling_a)) == 0);
+		a = __gem_mmap__gtt(fd, bo, huge_object_size, PROT_READ | PROT_WRITE);
+		igt_require(a);
+		gem_close(fd, bo);
 
-	bo = gem_create(fd, huge_object_size);
-	if (tiling_a)
-		igt_require(__gem_set_tiling(fd, bo, tiling_a, min_tile_width(devid, tiling_a)) == 0);
-	a = __gem_mmap__gtt(fd, bo, huge_object_size, PROT_READ | PROT_WRITE);
-	igt_require(a);
-	gem_close(fd, bo);
+		for (i = 0; i < huge_object_size / PAGE_SIZE; i++) {
+			uint32_t *ptr = (uint32_t *)(a + PAGE_SIZE*i);
+			for (int j = 0; j < PAGE_SIZE/4; j++)
+				ptr[j] = i + j;
+			igt_progress("Writing a ", i, huge_object_size / PAGE_SIZE);
+		}
 
-	for (i = 0; i < huge_object_size / PAGE_SIZE; i++) {
-		memcpy(a + PAGE_SIZE*i, pattern_a, PAGE_SIZE);
-		igt_progress("Writing a ", i, huge_object_size / PAGE_SIZE);
+		bo = gem_create(fd, huge_object_size);
+		if (tiling_b)
+			igt_require(__gem_set_tiling(fd, bo, abs(tiling_b), max_tile_width(devid, tiling_b)) == 0);
+		b = __gem_mmap__gtt(fd, bo, huge_object_size, PROT_READ | PROT_WRITE);
+		igt_require(b);
+		gem_close(fd, bo);
+
+		for (i = 0; i < huge_object_size / PAGE_SIZE; i++) {
+			uint32_t *ptr = (uint32_t *)(b + PAGE_SIZE*i);
+			for (int j = 0; j < PAGE_SIZE/4; j++)
+				ptr[j] = ~(i + j);
+			igt_progress("Writing b ", i, huge_object_size / PAGE_SIZE);
+		}
+
+		for (i = 0; i < huge_object_size / PAGE_SIZE; i++) {
+			uint32_t *A = (uint32_t *)(a + PAGE_SIZE*i);
+			uint32_t *B = (uint32_t *)(b + PAGE_SIZE*i);
+			uint32_t A_tmp[PAGE_SIZE/sizeof(uint32_t)];
+			uint32_t B_tmp[PAGE_SIZE/sizeof(uint32_t)];
+
+			copy_wc_page(A_tmp, A);
+			copy_wc_page(B_tmp, B);
+			for (int j = 0; j < PAGE_SIZE/4; j++)
+				if ((i +  j) & 1)
+					A_tmp[j] = B_tmp[j];
+				else
+					B_tmp[j] = A_tmp[j];
+			memcpy(A, A_tmp, PAGE_SIZE);
+			memcpy(B, B_tmp, PAGE_SIZE);
+
+			igt_progress("Copying a<->b ", i, huge_object_size / PAGE_SIZE);
+		}
+
+		for (i = 0; i < huge_object_size / PAGE_SIZE; i++) {
+			uint32_t page[PAGE_SIZE/sizeof(uint32_t)];
+			copy_wc_page(page, a + PAGE_SIZE*i);
+			for (int j = 0; j < PAGE_SIZE/sizeof(uint32_t); j++)
+				if ((i + j) & 1)
+					igt_assert_eq_u32(page[j], ~(i + j));
+				else
+					igt_assert_eq_u32(page[j], i + j);
+			igt_progress("Checking a ", i, huge_object_size / PAGE_SIZE);
+		}
+		munmap(a, huge_object_size);
+
+		for (i = 0; i < huge_object_size / PAGE_SIZE; i++) {
+			uint32_t page[PAGE_SIZE/sizeof(uint32_t)];
+			copy_wc_page(page, b + PAGE_SIZE*i);
+			for (int j = 0; j < PAGE_SIZE/sizeof(uint32_t); j++)
+				if ((i + j) & 1)
+					igt_assert_eq_u32(page[j], ~(i + j));
+				else
+					igt_assert_eq_u32(page[j], i + j);
+			igt_progress("Checking b ", i, huge_object_size / PAGE_SIZE);
+		}
+		munmap(b, huge_object_size);
 	}
-
-	bo = gem_create(fd, huge_object_size);
-	if (tiling_b)
-		igt_require(__gem_set_tiling(fd, bo, tiling_b, max_tile_width(devid, tiling_b)) == 0);
-	b = __gem_mmap__gtt(fd, bo, huge_object_size, PROT_READ | PROT_WRITE);
-	igt_require(b);
-	gem_close(fd, bo);
-
-	for (i = 0; i < huge_object_size / PAGE_SIZE; i++) {
-		memcpy(b + PAGE_SIZE*i, pattern_b, PAGE_SIZE);
-		igt_progress("Writing b ", i, huge_object_size / PAGE_SIZE);
-	}
-
-	for (i = 0; i < huge_object_size / PAGE_SIZE; i++) {
-		if (i & 1)
-			memcpy(a + i *PAGE_SIZE, b + i*PAGE_SIZE, PAGE_SIZE);
-		else
-			memcpy(b + i *PAGE_SIZE, a + i*PAGE_SIZE, PAGE_SIZE);
-		igt_progress("Copying a<->b ", i, huge_object_size / PAGE_SIZE);
-	}
-
-	for (i = 0; i < huge_object_size / PAGE_SIZE; i++) {
-		if (i & 1)
-			igt_assert(memcmp(pattern_b, a + PAGE_SIZE*i, PAGE_SIZE) == 0);
-		else
-			igt_assert(memcmp(pattern_a, a + PAGE_SIZE*i, PAGE_SIZE) == 0);
-		igt_progress("Checking a ", i, huge_object_size / PAGE_SIZE);
-	}
-	munmap(a, huge_object_size);
-
-	for (i = 0; i < huge_object_size / PAGE_SIZE; i++) {
-		if (i & 1)
-			igt_assert(memcmp(pattern_b, b + PAGE_SIZE*i, PAGE_SIZE) == 0);
-		else
-			igt_assert(memcmp(pattern_a, b + PAGE_SIZE*i, PAGE_SIZE) == 0);
-		igt_progress("Checking b ", i, huge_object_size / PAGE_SIZE);
-	}
-	munmap(b, huge_object_size);
-
-	free(pattern_a);
-	free(pattern_b);
+	igt_waitchildren();
 }
 
 static void
@@ -687,22 +773,47 @@ igt_main
 	igt_subtest("huge-bo-tiledY")
 		test_huge_bo(fd, 1, I915_TILING_Y);
 
-	igt_subtest("basic-small-copy")
-		test_huge_copy(fd, -2, I915_TILING_NONE, I915_TILING_NONE);
-	igt_subtest("basic-small-copy-XY")
-		test_huge_copy(fd, -2, I915_TILING_X, I915_TILING_Y);
-	igt_subtest("medium-copy")
-		test_huge_copy(fd, -1, I915_TILING_NONE, I915_TILING_NONE);
-	igt_subtest("medium-copy-XY")
-		test_huge_copy(fd, -1, I915_TILING_X, I915_TILING_Y);
-	igt_subtest("big-copy")
-		test_huge_copy(fd, 0, I915_TILING_NONE, I915_TILING_NONE);
-	igt_subtest("big-copy-XY")
-		test_huge_copy(fd, 0, I915_TILING_X, I915_TILING_Y);
-	igt_subtest("huge-copy")
-		test_huge_copy(fd, 1, I915_TILING_NONE, I915_TILING_NONE);
-	igt_subtest("huge-copy-XY")
-		test_huge_copy(fd, 1, I915_TILING_X, I915_TILING_Y);
+	igt_subtest_group {
+		const struct copy_size {
+			const char *prefix;
+			int size;
+		} copy_sizes[] = {
+			{ "basic-small", -2 },
+			{ "medium", -1 },
+			{ "big", 0 },
+			{ "huge", 1 },
+			{ "swap", 2 },
+			{ }
+		};
+		const struct copy_mode {
+			const char *suffix;
+			int tiling_x, tiling_y;
+		} copy_modes[] = {
+			{ "", I915_TILING_NONE, I915_TILING_NONE},
+			{ "-XY", I915_TILING_X, I915_TILING_Y},
+			{ "-odd", -I915_TILING_X, -I915_TILING_Y},
+			{}
+		};
+		const int ncpus = sysconf(_SC_NPROCESSORS_ONLN);
+
+		for (const struct copy_size *s = copy_sizes; s->prefix; s++)
+			for (const struct copy_mode *m = copy_modes; m->suffix; m++) {
+				igt_subtest_f("%s-copy%s", s->prefix, m->suffix)
+					test_huge_copy(fd,
+							s->size,
+							m->tiling_x,
+							m->tiling_y,
+							1);
+
+				igt_subtest_f("forked-%s-copy%s", s->prefix, m->suffix)
+					test_huge_copy(fd,
+							s->size,
+							m->tiling_x,
+							m->tiling_y,
+							ncpus);
+			}
+	}
+
 
 	igt_fixture
 		close(fd);
