@@ -27,6 +27,7 @@
  */
 
 #include "igt.h"
+#include "igt_sysfs.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -48,39 +49,19 @@ igt_render_copyfunc_t rendercopy;
 struct intel_batchbuffer *batch;
 int width, height;
 
-static int gem_param(int fd, int name)
-{
-	drm_i915_getparam_t gp;
-	int v = -1; /* No param uses the sign bit, reserve it for errors */
-
-	memset(&gp, 0, sizeof(gp));
-	gp.param = name;
-	gp.value = &v;
-	if (drmIoctl(fd, DRM_IOCTL_I915_GETPARAM, &gp))
-		return -1;
-
-	return v;
-}
-
 static int semaphores_enabled(int fd)
 {
-	FILE *file;
-	int detected = -1;
-	int ret;
+	bool enabled = false;
+	int dir;
 
-	ret = gem_param(fd, 20);
-	if (ret != -1)
-		return ret > 0;
+	dir = igt_sysfs_open_parameters(fd);
+	if (dir < 0)
+		return false;
 
-	file = fopen("/sys/module/i915/parameters/semaphores", "r");
-	if (file) {
-		int value;
-		if (fscanf(file, "%d", &value) == 1)
-			detected = value;
-		fclose(file);
-	}
+	enabled = igt_sysfs_get_boolean(dir, "semaphores");
+	close(dir);
 
-	return detected;
+	return enabled;
 }
 
 static drm_intel_bo *rcs_copy_bo(drm_intel_bo *dst, drm_intel_bo *src)
@@ -154,7 +135,8 @@ static drm_intel_bo *create_bo(drm_intel_bufmgr *bufmgr,
 					&tiling_mode, &pitch, 0);
 }
 
-static void run(drm_intel_bufmgr *bufmgr, int _width, int _height)
+static void run(drm_intel_bufmgr *bufmgr, int _width, int _height,
+		bool write_bcs, bool write_rcs)
 {
 	drm_intel_bo *src = NULL, *bcs = NULL, *rcs = NULL;
 	drm_intel_bo *bcs_batch, *rcs_batch;
@@ -170,8 +152,16 @@ static void run(drm_intel_bufmgr *bufmgr, int _width, int _height)
 
 	set_bo(src, 0xdeadbeef);
 
-	rcs_batch = rcs_copy_bo(rcs, src);
-	bcs_batch = bcs_copy_bo(bcs, src);
+	if (write_bcs) {
+		bcs_batch = bcs_copy_bo(src, bcs);
+	} else {
+		bcs_batch = bcs_copy_bo(bcs, src);
+	}
+	if (write_rcs) {
+		rcs_batch = rcs_copy_bo(src, rcs);
+	} else {
+		rcs_batch = rcs_copy_bo(rcs, src);
+	}
 
 	drm_intel_bo_unreference(rcs);
 	drm_intel_bo_unreference(bcs);
@@ -185,7 +175,9 @@ static void run(drm_intel_bufmgr *bufmgr, int _width, int _height)
 	drm_intel_gem_bo_start_gtt_access(src, true);
 	clock_gettime(CLOCK_MONOTONIC, &end);
 
-	igt_info("Time to read-read %dx%d [%dk]:		%7.3fµs\n",
+	igt_info("Time to %s-%s %dx%d [%dk]:		%7.3fµs\n",
+		 write_bcs ? "write" : "read",
+		 write_rcs ? "write" : "read",
 		 width, height, 4*width*height/1024,
 		 elapsed(&start, &end, loops));
 
@@ -222,7 +214,14 @@ igt_main
 		igt_info("Semaphores: %d\n", semaphores_enabled(fd));
 	}
 
-	for (i = 0; sizes[i] != 0; i++)
+	for (i = 0; sizes[i] != 0; i++) {
 		igt_subtest_f("read-read-%dx%d", sizes[i], sizes[i])
-			run(bufmgr, sizes[i], sizes[i]);
+			run(bufmgr, sizes[i], sizes[i], false, false);
+		igt_subtest_f("read-write-%dx%d", sizes[i], sizes[i])
+			run(bufmgr, sizes[i], sizes[i], false, true);
+		igt_subtest_f("write-read-%dx%d", sizes[i], sizes[i])
+			run(bufmgr, sizes[i], sizes[i], true, false);
+		igt_subtest_f("write-write-%dx%d", sizes[i], sizes[i])
+			run(bufmgr, sizes[i], sizes[i], true, true);
+	}
 }
