@@ -148,13 +148,17 @@ static int gem_reset_status(int fd, int ctx_id)
 	return RS_NO_ERROR;
 }
 
+static struct timespec ts_injected;
+
 #define BAN HANG_ALLOW_BAN
 #define ASYNC 2
 static void inject_hang(int fd, uint32_t ctx,
 			const struct intel_execution_engine *e,
 			unsigned flags)
 {
-	igt_hang_ring_t hang;
+	igt_hang_t hang;
+
+	clock_gettime(CLOCK_MONOTONIC, &ts_injected);
 
 	hang = igt_hang_ctx(fd, ctx, e->exec_id | e->flags, flags & BAN, NULL);
 	if ((flags & ASYNC) == 0)
@@ -238,6 +242,8 @@ static void test_rs(const struct intel_execution_engine *e,
 			assert_reset_status(i, fd[i], 0, RS_BATCH_PENDING);
 	}
 
+	igt_assert(igt_seconds_elapsed(&ts_injected) <= 30);
+
 	for (i = 0; i < num_fds; i++)
 		close(fd[i]);
 }
@@ -290,6 +296,8 @@ static void test_rs_ctx(const struct intel_execution_engine *e,
 	}
 	sync_gpu();
 
+	igt_assert(igt_seconds_elapsed(&ts_injected) <= 30);
+
 	for (i = 0; i < num_fds; i++)
 		assert_reset_status(i, fd[i], 0, RS_NO_ERROR);
 
@@ -344,7 +352,7 @@ static void test_ban(const struct intel_execution_engine *e)
 
 	/* The second hang will count as pending and be discarded */
 	active_count--;
-	pending_count += 2; /* inject hang does 2 execs (query, then hang) */
+	pending_count += 1; /* inject hang does 1 real exec + 1 dummy */
 	while (retry--) {
 		inject_hang(fd_bad, 0, e, BAN);
 		active_count++;
@@ -539,7 +547,7 @@ static void test_close_pending_fork(const struct intel_execution_engine *e,
 				    const bool reverse)
 {
 	int fd = drm_open_driver(DRIVER_INTEL);
-	igt_hang_ring_t hang;
+	igt_hang_t hang;
 	int pid;
 
 	assert_reset_status(fd, fd, 0, RS_NO_ERROR);
