@@ -51,9 +51,11 @@ static void invalid_tests(int fd)
 	f.pitches[0] = 512*4;
 
 	igt_fixture {
-		gem_bo = gem_create(fd, 1024*1024*4);
+		gem_bo = igt_create_bo_with_dimensions(fd, 1024, 1024,
+			DRM_FORMAT_XRGB8888, 0, 0, NULL, NULL, NULL);
 		igt_assert(gem_bo);
-		gem_bo_small = gem_create(fd, 1024*1024*4 - 4096);
+		gem_bo_small = igt_create_bo_with_dimensions(fd, 1024, 1023,
+			DRM_FORMAT_XRGB8888, 0, 0, NULL, NULL, NULL);
 		igt_assert(gem_bo_small);
 
 		f.handles[0] = gem_bo;
@@ -129,7 +131,8 @@ static void pitch_tests(int fd)
 	f.pitches[0] = 1024*4;
 
 	igt_fixture {
-		gem_bo = gem_create(fd, 1024*1024*4);
+		gem_bo = igt_create_bo_with_dimensions(fd, 1024, 1024,
+			DRM_FORMAT_XRGB8888, 0, 0, NULL, NULL, NULL);
 		igt_assert(gem_bo);
 	}
 
@@ -153,42 +156,81 @@ static void pitch_tests(int fd)
 		}
 	}
 
-	igt_fixture
-		gem_set_tiling(fd, gem_bo, I915_TILING_X, 1024*4);
-	f.pitches[0] = 1024*4;
-
-	igt_subtest("basic-X-tiled") {
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == 0);
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
-		f.fb_id = 0;
-	}
-
-	igt_subtest("framebuffer-vs-set-tiling") {
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == 0);
-		igt_assert(__gem_set_tiling(fd, gem_bo, I915_TILING_X, 512*4) == -EBUSY);
-		igt_assert(__gem_set_tiling(fd, gem_bo, I915_TILING_X, 1024*4) == -EBUSY);
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
-		f.fb_id = 0;
-	}
-
-	f.pitches[0] = 512*4;
-	igt_subtest("tile-pitch-mismatch") {
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == -1 &&
-			   errno == EINVAL);
-	}
-
-	igt_fixture
-		gem_set_tiling(fd, gem_bo, I915_TILING_Y, 1024*4);
-	f.pitches[0] = 1024*4;
-	igt_subtest("basic-Y-tiled") {
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == -1 &&
-			   errno == EINVAL);
-	}
-
 	igt_fixture
 		gem_close(fd, gem_bo);
 }
 
+static void tiling_tests(int fd)
+{
+	struct drm_mode_fb_cmd2 f = {};
+	uint32_t tiled_x_bo = 0;
+	uint32_t tiled_y_bo = 0;
+
+	f.width = 512;
+	f.height = 512;
+	f.pixel_format = DRM_FORMAT_XRGB8888;
+	f.pitches[0] = 1024*4;
+
+	igt_subtest_group {
+		igt_fixture {
+			tiled_x_bo = igt_create_bo_with_dimensions(fd, 1024, 1024,
+				DRM_FORMAT_XRGB8888, LOCAL_I915_FORMAT_MOD_X_TILED,
+				1024*4, NULL, NULL, NULL);
+			igt_assert(tiled_x_bo);
+
+			tiled_y_bo = igt_create_bo_with_dimensions(fd, 1024, 1024,
+				DRM_FORMAT_XRGB8888, LOCAL_I915_FORMAT_MOD_Y_TILED,
+				1024*4, NULL, NULL, NULL);
+			igt_assert(tiled_y_bo);
+
+			gem_bo = igt_create_bo_with_dimensions(fd, 1024, 1024,
+				DRM_FORMAT_XRGB8888, 0, 0, NULL, NULL, NULL);
+			igt_assert(gem_bo);
+		}
+
+		f.pitches[0] = 1024*4;
+		igt_subtest("basic-X-tiled") {
+			f.handles[0] = tiled_x_bo;
+
+			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == 0);
+			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
+			f.fb_id = 0;
+		}
+
+		igt_subtest("framebuffer-vs-set-tiling") {
+			f.handles[0] = gem_bo;
+
+			gem_set_tiling(fd, gem_bo, I915_TILING_X, 1024*4);
+			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == 0);
+			igt_assert(__gem_set_tiling(fd, gem_bo, I915_TILING_X, 512*4) == -EBUSY);
+			igt_assert(__gem_set_tiling(fd, gem_bo, I915_TILING_X, 1024*4) == -EBUSY);
+			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
+			f.fb_id = 0;
+		}
+
+		f.pitches[0] = 512*4;
+		igt_subtest("tile-pitch-mismatch") {
+			f.handles[0] = tiled_x_bo;
+
+			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == -1 &&
+				   errno == EINVAL);
+		}
+
+		f.pitches[0] = 1024*4;
+		igt_subtest("basic-Y-tiled") {
+			f.handles[0] = tiled_y_bo;
+
+			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == -1 &&
+				   errno == EINVAL);
+		}
+
+		igt_fixture {
+			gem_close(fd, tiled_x_bo);
+			gem_close(fd, tiled_y_bo);
+		}
+	}
+}
+
 static void size_tests(int fd)
 {
 	struct drm_mode_fb_cmd2 f = {};
@@ -211,9 +253,11 @@ static void size_tests(int fd)
 	f_8.pitches[0] = 1024*2;
 
 	igt_fixture {
-		gem_bo = gem_create(fd, 1024*1024*4);
+		gem_bo = igt_create_bo_with_dimensions(fd, 1024, 1024,
+			DRM_FORMAT_XRGB8888, 0, 0, NULL, NULL, NULL);
 		igt_assert(gem_bo);
-		gem_bo_small = gem_create(fd, 1024*1024*4 - 4096);
+		gem_bo_small = igt_create_bo_with_dimensions(fd, 1024, 1023,
+			DRM_FORMAT_XRGB8888, 0, 0, NULL, NULL, NULL);
 		igt_assert(gem_bo_small);
 	}
 
@@ -273,10 +317,8 @@ static void size_tests(int fd)
 		f.fb_id = 0;
 	}
 
-	igt_fixture
-		gem_set_tiling(fd, gem_bo_small, I915_TILING_X, 1024*4);
-
 	igt_subtest("bo-too-small-due-to-tiling") {
+		gem_set_tiling(fd, gem_bo_small, I915_TILING_X, 1024*4);
 		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == -1 &&
 			   errno == EINVAL);
 	}
@@ -293,7 +335,8 @@ static void addfb25_tests(int fd)
 	struct local_drm_mode_fb_cmd2 f = {};
 
 	igt_fixture {
-		gem_bo = gem_create(fd, 1024*1024*4);
+		gem_bo = igt_create_bo_with_dimensions(fd, 1024, 1024,
+			DRM_FORMAT_XRGB8888, 0, 0, NULL, NULL, NULL);
 		igt_assert(gem_bo);
 
 		memset(&f, 0, sizeof(f));
@@ -324,53 +367,61 @@ static void addfb25_tests(int fd)
 		igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) < 0 && errno == EINVAL);
 	}
 
-	igt_fixture
-		gem_set_tiling(fd, gem_bo, I915_TILING_X, 1024*4);
+	igt_subtest_group {
+		igt_fixture {
+			gem_set_tiling(fd, gem_bo, I915_TILING_X, 1024*4);
+			igt_require_fb_modifiers(fd);
+		}
 
-	igt_subtest("addfb25-X-tiled-mismatch") {
-		igt_require_fb_modifiers(fd);
+		igt_subtest("addfb25-X-tiled-mismatch") {
+			f.modifier[0] = LOCAL_DRM_FORMAT_MOD_NONE;
+			igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) < 0 && errno == EINVAL);
+		}
 
-		f.modifier[0] = LOCAL_DRM_FORMAT_MOD_NONE;
-		igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) < 0 && errno == EINVAL);
+		igt_subtest("addfb25-X-tiled") {
+			f.modifier[0] = LOCAL_I915_FORMAT_MOD_X_TILED;
+			igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) == 0);
+			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
+			f.fb_id = 0;
+		}
+
+		igt_subtest("addfb25-framebuffer-vs-set-tiling") {
+			f.modifier[0] = LOCAL_I915_FORMAT_MOD_X_TILED;
+			igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) == 0);
+			igt_assert(__gem_set_tiling(fd, gem_bo, I915_TILING_X, 512*4) == -EBUSY);
+			igt_assert(__gem_set_tiling(fd, gem_bo, I915_TILING_X, 1024*4) == -EBUSY);
+			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
+			f.fb_id = 0;
+		}
 	}
-
-	igt_subtest("addfb25-X-tiled") {
-		igt_require_fb_modifiers(fd);
-
-		f.modifier[0] = LOCAL_I915_FORMAT_MOD_X_TILED;
-		igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) == 0);
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
-		f.fb_id = 0;
-	}
-
-	igt_subtest("addfb25-framebuffer-vs-set-tiling") {
-		igt_require_fb_modifiers(fd);
-
-		f.modifier[0] = LOCAL_I915_FORMAT_MOD_X_TILED;
-		igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) == 0);
-		igt_assert(__gem_set_tiling(fd, gem_bo, I915_TILING_X, 512*4) == -EBUSY);
-		igt_assert(__gem_set_tiling(fd, gem_bo, I915_TILING_X, 1024*4) == -EBUSY);
-		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
-		f.fb_id = 0;
-	}
-
 	igt_fixture
 		gem_close(fd, gem_bo);
 }
 
-static void addfb25_ytile(int fd, int gen)
+static int addfb_expected_ret(int fd)
+{
+	int gen;
+
+	if (!is_i915_device(fd))
+		return 0;
+
+	gen = intel_gen(intel_get_drm_devid(fd));
+	return gen >= 9 ? 0 : -1;
+}
+
+static void addfb25_ytile(int fd)
 {
 	struct local_drm_mode_fb_cmd2 f = {};
-	int shouldret;
+	int gen;
 
 	igt_fixture {
-		gem_bo = gem_create(fd, 1024*1024*4);
+		gem_bo = igt_create_bo_with_dimensions(fd, 1024, 1024,
+			DRM_FORMAT_XRGB8888, 0, 0, NULL, NULL, NULL);
 		igt_assert(gem_bo);
-		gem_bo_small = gem_create(fd, 1024*1023*4);
+		gem_bo_small = igt_create_bo_with_dimensions(fd, 1024, 1023,
+			DRM_FORMAT_XRGB8888, 0, 0, NULL, NULL, NULL);
 		igt_assert(gem_bo_small);
 
-		shouldret = gen >= 9 ? 0 : -1;
-
 		memset(&f, 0, sizeof(f));
 
 		f.width = 1024;
@@ -387,8 +438,9 @@ static void addfb25_ytile(int fd, int gen)
 		igt_require_fb_modifiers(fd);
 
 		f.modifier[0] = LOCAL_I915_FORMAT_MOD_Y_TILED;
-		igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) == shouldret);
-		if (!shouldret)
+		igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) ==
+			   addfb_expected_ret(fd));
+		if (!addfb_expected_ret(fd))
 			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
 		f.fb_id = 0;
 	}
@@ -397,14 +449,17 @@ static void addfb25_ytile(int fd, int gen)
 		igt_require_fb_modifiers(fd);
 
 		f.modifier[0] = LOCAL_I915_FORMAT_MOD_Yf_TILED;
-		igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) == shouldret);
-		if (!shouldret)
+		igt_assert(drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f) ==
+			   addfb_expected_ret(fd));
+		if (!addfb_expected_ret(fd))
 			igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
 		f.fb_id = 0;
 	}
 
 	igt_subtest("addfb25-Y-tiled-small") {
 		igt_require_fb_modifiers(fd);
+
+		gen = intel_gen(intel_get_drm_devid(fd));
 		igt_require(gen >= 9);
 
 		f.modifier[0] = LOCAL_I915_FORMAT_MOD_Y_TILED;
@@ -420,15 +475,76 @@ static void addfb25_ytile(int fd, int gen)
 	}
 }
 
+static void prop_tests(int fd)
+{
+	struct drm_mode_fb_cmd2 f = {};
+	struct drm_mode_obj_get_properties get_props = {};
+	struct drm_mode_obj_set_property set_prop = {};
+	uint64_t prop, prop_val;
+
+	f.width = 1024;
+	f.height = 1024;
+	f.pixel_format = DRM_FORMAT_XRGB8888;
+	f.pitches[0] = 1024*4;
+
+	igt_fixture {
+		gem_bo = igt_create_bo_with_dimensions(fd, 1024, 1024,
+			DRM_FORMAT_XRGB8888, 0, 0, NULL, NULL, NULL);
+		igt_assert(gem_bo);
+
+		f.handles[0] = gem_bo;
+
+		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_ADDFB2, &f) == 0);
+	}
+
+	get_props.props_ptr = (uintptr_t) &prop;
+	get_props.prop_values_ptr = (uintptr_t) &prop_val;
+	get_props.count_props = 1;
+	get_props.obj_id = f.fb_id;
+
+	igt_subtest("invalid-get-prop-any") {
+		get_props.obj_type = 0; /* DRM_MODE_OBJECT_ANY */
+
+		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_OBJ_GETPROPERTIES,
+				    &get_props) == -1 && errno == EINVAL);
+	}
+
+	igt_subtest("invalid-get-prop") {
+		get_props.obj_type = DRM_MODE_OBJECT_FB;
+
+		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_OBJ_GETPROPERTIES,
+				    &get_props) == -1 && errno == EINVAL);
+	}
+
+	set_prop.value = 0;
+	set_prop.prop_id = 1;
+	set_prop.obj_id = f.fb_id;
+
+	igt_subtest("invalid-set-prop-any") {
+		set_prop.obj_type = 0; /* DRM_MODE_OBJECT_ANY */
+
+		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_OBJ_SETPROPERTY,
+				    &set_prop) == -1 && errno == EINVAL);
+	}
+
+	igt_subtest("invalid-set-prop") {
+		set_prop.obj_type = DRM_MODE_OBJECT_FB;
+
+		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_OBJ_SETPROPERTY,
+				    &set_prop) == -1 && errno == EINVAL);
+	}
+
+	igt_fixture
+		igt_assert(drmIoctl(fd, DRM_IOCTL_MODE_RMFB, &f.fb_id) == 0);
+
+}
+
 int fd;
-int gen;
 
 igt_main
 {
-	igt_fixture {
-		fd = drm_open_driver_master(DRIVER_INTEL);
-		gen = intel_gen(intel_get_drm_devid(fd));
-	}
+	igt_fixture
+		fd = drm_open_driver_master(DRIVER_ANY);
 
 	invalid_tests(fd);
 
@@ -438,7 +554,11 @@ igt_main
 
 	addfb25_tests(fd);
 
-	addfb25_ytile(fd, gen);
+	addfb25_ytile(fd);
+
+	tiling_tests(fd);
+
+	prop_tests(fd);
 
 	igt_fixture
 		close(fd);
