@@ -43,6 +43,10 @@
 #include "intel_aub.h"
 #include "intel_chipset.h"
 
+#ifndef ARRAY_SIZE
+#define ARRAY_SIZE(x) (sizeof(x)/sizeof((x)[0]))
+#endif
+
 static int close_init_helper(int fd);
 static int ioctl_init_helper(int fd, unsigned long request, ...);
 
@@ -50,11 +54,10 @@ static int (*libc_close)(int fd) = close_init_helper;
 static int (*libc_ioctl)(int fd, unsigned long request, ...) = ioctl_init_helper;
 
 static int drm_fd = -1;
-static char *filename;
-static FILE *file;
+static char *filename = NULL;
+static FILE *files[2] = { NULL, NULL };
 static int gen = 0;
 static int verbose = 0;
-static const uint32_t gtt_size = 0x10000;
 static bool device_override;
 static uint32_t device;
 
@@ -140,42 +143,85 @@ align_u64(uint64_t v, uint64_t a)
 static void
 dword_out(uint32_t data)
 {
-	fwrite(&data, 1, 4, file);
+	for (int i = 0; i < ARRAY_SIZE (files); i++) {
+		if (files[i] == NULL)
+			continue;
+
+		fail_if(fwrite(&data, 1, 4, files[i]) == 0,
+			"Writing to output failed\n");
+	}
 }
 
 static void
 data_out(const void *data, size_t size)
 {
-	fwrite(data, 1, size, file);
+	if (size == 0)
+		return;
+
+	for (int i = 0; i < ARRAY_SIZE (files); i++) {
+		if (files[i] == NULL)
+			continue;
+
+		fail_if(fwrite(data, 1, size, files[i]) == 0,
+			"Writing to output failed\n");
+	}
+}
+
+static uint32_t
+gtt_entry_size(void)
+{
+	return gen >= 8 ? 8 : 4;
+}
+
+static uint32_t
+gtt_size(void)
+{
+	/* Enough for 64MB assuming 4kB pages. */
+	const unsigned entries = 0x4000;
+	return entries * gtt_entry_size();
 }
 
 static void
 write_header(void)
 {
+	char app_name[8 * 4];
+	char comment[16];
+	int comment_len, comment_dwords, dwords;
 	uint32_t entry = 0x200003;
 
+	comment_len = snprintf(comment, sizeof(comment), "PCI-ID=0x%x", device);
+	comment_dwords = ((comment_len + 3) / 4);
+
 	/* Start with a (required) version packet. */
-	dword_out(CMD_AUB_HEADER | (13 - 2));
+	dwords = 13 + comment_dwords;
+	dword_out(CMD_AUB_HEADER | (dwords - 2));
 	dword_out((4 << AUB_HEADER_MAJOR_SHIFT) |
 		  (0 << AUB_HEADER_MINOR_SHIFT));
-	for (int i = 0; i < 8; i++)
-		dword_out(0); /* app name */
+
+	/* Next comes a 32-byte application name. */
+	strncpy(app_name, program_invocation_short_name, sizeof(app_name));
+	app_name[sizeof(app_name) - 1] = 0;
+	data_out(app_name, sizeof(app_name));
 
 	dword_out(0); /* timestamp */
 	dword_out(0); /* timestamp */
-	dword_out(0); /* comment len */
+	dword_out(comment_len);
+	data_out(comment, comment_dwords * 4);
 
-	/* Set up the GTT. The max we can handle is 256M */
+	/* Set up the GTT. The max we can handle is 64M */
 	dword_out(CMD_AUB_TRACE_HEADER_BLOCK | ((gen >= 8 ? 6 : 5) - 2));
 	dword_out(AUB_TRACE_MEMTYPE_GTT_ENTRY |
 		  AUB_TRACE_TYPE_NOTYPE | AUB_TRACE_OP_DATA_WRITE);
 	dword_out(0); /* subtype */
 	dword_out(0); /* offset */
-	dword_out(gtt_size); /* size */
+	dword_out(gtt_size()); /* size */
 	if (gen >= 8)
 		dword_out(0);
-	for (uint32_t i = 0; i < gtt_size; i += 4, entry += 0x1000)
-		dword_out(entry);
+	for (uint32_t i = 0; i * gtt_entry_size() < gtt_size(); i++) {
+		dword_out(entry + 0x1000 * i);
+		if (gen >= 8)
+			dword_out(0);
+	}
 }
 
 /**
@@ -254,6 +300,32 @@ aub_dump_ringbuffer(uint64_t batch_offset, uint64_t offset, int ring_flag)
 	data_out(ringbuffer, ring_count * 4);
 }
 
+static void
+write_reloc(void *p, uint64_t v)
+{
+	if (gen >= 8) {
+		/* From the Broadwell PRM Vol. 2a,
+		 * MI_LOAD_REGISTER_MEM::MemoryAddress:
+		 *
+		 *	"This field specifies the address of the memory
+		 *	location where the register value specified in the
+		 *	DWord above will read from.  The address specifies
+		 *	the DWord location of the data. Range =
+		 *	GraphicsVirtualAddress[63:2] for a DWord register
+		 *	GraphicsAddress [63:48] are ignored by the HW and
+		 *	assumed to be in correct canonical form [63:48] ==
+		 *	[47]."
+		 *
+		 * In practice, this will always mean the top bits are zero
+		 * because of the GTT size limitation of the aubdump tool.
+		 */
+		const int shift = 63 - 47;
+		*(uint64_t *)p = (((int64_t)v) << shift) >> shift;
+	} else {
+		*(uint32_t *)p = v;
+	}
+}
+
 static void *
 relocate_bo(struct bo *bo, const struct drm_i915_gem_execbuffer2 *execbuffer2,
 	    const struct drm_i915_gem_exec_object2 *obj)
@@ -263,7 +335,6 @@ relocate_bo(struct bo *bo, const struct drm_i915_gem_execbuffer2 *execbuffer2,
 	const struct drm_i915_gem_relocation_entry *relocs =
 		(const struct drm_i915_gem_relocation_entry *) (uintptr_t) obj->relocs_ptr;
 	void *relocated;
-	uint32_t *dw;
 	int handle;
 
 	relocated = malloc(bo->size);
@@ -277,8 +348,8 @@ relocate_bo(struct bo *bo, const struct drm_i915_gem_execbuffer2 *execbuffer2,
 		else
 			handle = relocs[i].target_handle;
 
-		dw = (uint32_t*)(((char *) relocated) + relocs[i].offset);
-		*dw = get_bo(handle)->offset + relocs[i].delta;
+		write_reloc(((char *)relocated) + relocs[i].offset,
+			    get_bo(handle)->offset + relocs[i].delta);
 	}
 
 	return relocated;
@@ -332,7 +403,7 @@ dump_execbuffer2(int fd, struct drm_i915_gem_execbuffer2 *execbuffer2)
 	struct drm_i915_gem_exec_object2 *exec_objects =
 		(struct drm_i915_gem_exec_object2 *) (uintptr_t) execbuffer2->buffers_ptr;
 	uint32_t ring_flag = execbuffer2->flags & I915_EXEC_RING_MASK;
-	uint32_t offset = gtt_size;
+	uint32_t offset = gtt_size();
 	struct drm_i915_gem_exec_object2 *obj;
 	struct bo *bo, *batch_bo;
 	void *data;
@@ -393,7 +464,10 @@ dump_execbuffer2(int fd, struct drm_i915_gem_execbuffer2 *execbuffer2)
 	aub_dump_ringbuffer(batch_bo->offset + execbuffer2->batch_start_offset,
 			    offset, ring_flag);
 
-	fflush(file);
+	for (int i = 0; i < ARRAY_SIZE(files); i++) {
+		if (files[i] != NULL)
+			fflush(files[i]);
+	}
 }
 
 static void
@@ -426,6 +500,85 @@ close(int fd)
 	return libc_close(fd);
 }
 
+static FILE *
+launch_command(char *command)
+{
+	int i = 0, fds[2];
+	char **args = calloc(strlen(command), sizeof(char *));
+	char *iter = command;
+
+	args[i++] = iter = command;
+
+	while ((iter = strstr(iter, ",")) != NULL) {
+		*iter = '\0';
+		iter += 1;
+		args[i++] = iter;
+	}
+
+	if (pipe(fds) == -1)
+		return NULL;
+
+	switch (fork()) {
+	case 0:
+		dup2(fds[0], 0);
+		fail_if(execvp(args[0], args) == -1,
+			"intel_aubdump: failed to launch child command\n");
+		return NULL;
+
+	default:
+		free(args);
+		return fdopen(fds[1], "w");
+
+	case -1:
+		return NULL;
+	}
+}
+
+static void
+maybe_init(void)
+{
+	static bool initialized = false;
+	FILE *config;
+	char *key, *value;
+
+	if (initialized)
+		return;
+
+	initialized = true;
+
+	config = fdopen(3, "r");
+	while (fscanf(config, "%m[^=]=%m[^\n]\n", &key, &value) != EOF) {
+		if (!strcmp(key, "verbose")) {
+			verbose = 1;
+		} else if (!strcmp(key, "device")) {
+			fail_if(sscanf(value, "%i", &device) != 1,
+				"intel_aubdump: failed to parse device id '%s'",
+				value);
+			device_override = true;
+		} else if (!strcmp(key, "file")) {
+			filename = strdup(value);
+			files[0] = fopen(filename, "w+");
+			fail_if(files[0] == NULL,
+				"intel_aubdump: failed to open file '%s'\n",
+				filename);
+		} else if (!strcmp(key,  "command")) {
+			files[1] = launch_command(value);
+			fail_if(files[1] == NULL,
+				"intel_aubdump: failed to launch command '%s'\n",
+				value);
+		} else {
+			fprintf(stderr, "intel_aubdump: unknown option '%s'\n", key);
+		}
+
+		free(key);
+		free(value);
+	}
+	fclose(config);
+
+	bos = malloc(MAX_BO_COUNT * sizeof(bos[0]));
+	fail_if(bos == NULL, "intel_aubdump: out of memory\n");
+}
+
 int
 ioctl(int fd, unsigned long request, ...)
 {
@@ -447,6 +600,8 @@ ioctl(int fd, unsigned long request, ...)
 	}
 
 	if (fd == drm_fd) {
+		maybe_init();
+
 		switch (request) {
 		case DRM_IOCTL_I915_GETPARAM: {
 			struct drm_i915_getparam *getparam = argp;
@@ -550,26 +705,10 @@ ioctl(int fd, unsigned long request, ...)
 static void
 init(void)
 {
-	const char *args = getenv("INTEL_AUBDUMP_ARGS");
-
 	libc_close = dlsym(RTLD_NEXT, "close");
 	libc_ioctl = dlsym(RTLD_NEXT, "ioctl");
 	fail_if(libc_close == NULL || libc_ioctl == NULL,
 		"intel_aubdump: failed to get libc ioctl or close\n");
-
-	if (sscanf(args, "verbose=%d;file=%m[^;];device=%i",
-		   &verbose, &filename, &device) != 3)
-		filename = strdup("intel.aub");
-	fail_if(filename == NULL, "intel_aubdump: out of memory\n");
-
-	if (device)
-		device_override = true;
-
-	bos = malloc(MAX_BO_COUNT * sizeof(bos[0]));
-	fail_if(bos == NULL, "intel_aubdump: out of memory\n");
-
-	file = fopen(filename, "w+");
-	fail_if(file == NULL, "intel_aubdump: failed to open file '%s'\n", filename);
 }
 
 static int
@@ -597,7 +736,9 @@ static void __attribute__ ((destructor))
 fini(void)
 {
 	free(filename);
-	if (file)
-		fclose(file);
+	for (int i = 0; i < ARRAY_SIZE(files); i++) {
+		if (files[i] != NULL)
+			fclose(files[i]);
+	}
 	free(bos);
 }
