@@ -507,7 +507,7 @@ end:
 	return zstream.total_out / 4;
 }
 
-static int ascii85_decode(const char *in, uint32_t **out)
+static int ascii85_decode(const char *in, uint32_t **out, bool inflate)
 {
 	int len = 0, size = 1024;
 
@@ -538,6 +538,9 @@ static int ascii85_decode(const char *in, uint32_t **out)
 		(*out)[len++] = v;
 	}
 
+	if (!inflate)
+		return len;
+
 	return zlib_inflate(out, len);
 }
 
@@ -638,8 +641,8 @@ read_data_file(FILE *file)
 			}
 		}
 
-		if (line[0] == ':') {
-			count = ascii85_decode(line+1, &data);
+		if (line[0] == ':' || line[0] == '~') {
+			count = ascii85_decode(line+1, &data, line[0] == ':');
 			if (count == 0) {
 				fprintf(stderr, "ASCII85 decode failed.\n");
 				exit(1);
@@ -847,4 +850,4 @@ main(int argc, char *argv[])
 	return 0;
 }
 
-/* vim: set ts=8 sw=8 tw=0 noet :*/
+/* vim: set ts=8 sw=8 tw=0 cino=:0,(0 noet :*/
